/**
 *******************************************************************************
 * @file        a34m456_adc.h
 * @author      ABOV R&D Division
 * @brief       ADC driver
 *
 * Copyright 2024 ABOV Semiconductor Co.,Ltd. All rights reserved.
 *
 * This file is licensed under terms that are found in the LICENSE file
 * located at Document directory.
 * If this file is delivered or shared without applicable license terms,
 * the terms of the BSD-3-Clause license shall be applied.
 * Reference: https://opensource.org/licenses/BSD-3-Clause
 ******************************************************************************/

/**
 * @addtogroup  driver
 * @{
 * @defgroup    driver_adc ADC
 * @{
 * @brief       ADC driver
 */

#ifndef _A34M456_ADC_H_
#define _A34M456_ADC_H_

#ifdef __cplusplus
extern "C"
{
#endif

#include <stdint.h>
#include "a34m456_driver_common.h"
#include "a34m456_dma.h"

/** ADC Ready time-out definitions in case of using function
 *
 */
#define ADC_READY_TIMEOUT               (0x10UL)
#define ADC_SR_READY_TIMEOUT            (0xFFFFFFFFUL)
/*
 * These bits allow control over which A/D channels generate
 * interrupts for conversion completion
 */
#define ADC_INTEN_DMA                   ((1UL << 4))
#define ADC_INTEN_TRIGGER               ((1UL << 3))
#define ADC_INTEN_SEQUENCE              ((1UL << 2))
#define ADC_INTEN_SINGLE                ((1UL << 0))

#define ADC_STAT_SINGLE                 ((1UL << 0))

#define ADC_RAW_TO_VAL(x)               ((x >> 4) & 0x0FFF)
#define ADC_RAW_TO_CH(x)                ((x >> 16) & 0x1F)

/**
 * ADC id
 */
enum adc_id
{
    ADC_ID_0,                               /**< ADC 0 */
    ADC_ID_1,                               /**< ADC 1 */
    ADC_ID_2,                               /**< ADC 2 */
    ADC_ID_MAX,                             /**< ADC max */
};

/**
 * ADC Sequence DR id
 */
enum adc_sequence_id
{
    ADC_SEQUENCE_ID_0,                      /**< Sequence DR 0 data*/
    ADC_SEQUENCE_ID_1,                      /**< Sequence DR 1 data*/
    ADC_SEQUENCE_ID_2,                      /*!< Sequence DR 2 data*/
    ADC_SEQUENCE_ID_3,                      /*!< Sequence DR 3 data*/
    ADC_SEQUENCE_ID_4,                      /**< Sequence DR 4 data*/
    ADC_SEQUENCE_ID_5,                      /**< Sequence DR 5 data*/
    ADC_SEQUENCE_ID_6,                      /*!< Sequence DR 6 data*/
    ADC_SEQUENCE_ID_7,                      /*!< Sequence DR 7 data*/
    ADC_SEQUENCE_MAX,
};

/**
 * ADC mode
 */
enum adc_mode
{
    ADC_MODE_SINGLE,                        /**< adc single conversion mode*/
    ADC_MODE_SEQUNCE,                       /**< adc sequence conversion mode*/
    ADC_MODE_BURST,                         /**< adc burst conversion mode */
    ADC_MODE_MULTI,                         /*!< adc multiple conversion mode */
};

/**
 * ADC external request enable
 */
enum adc_ext_request
{
    ADC_EXTREQ_DISABLE,                      /**< Conversion start control request bit from another external ADC*/
    ADC_EXTREQ_ENABLE,                       /**< Conversion start control request bit from another external ADC*/
};

/**
 * ADC Sequence DR id
 */
enum adc_trigger_sequence_id
{
    ADC_TRIGGER_SEQUENCE_ID_BURST   = 0,     /**< Trigger Sequence 0 data*/
    ADC_TRIGGER_SEQUENCE_ID_0       = 0,     /**< Trigger Sequence 0 data*/
    ADC_TRIGGER_SEQUENCE_ID_1       = 1,     /**< Trigger Sequence 1 data*/
    ADC_TRIGGER_SEQUENCE_ID_2       = 2,     /*!< Trigger Sequence 2 data*/
    ADC_TRIGGER_SEQUENCE_ID_3       = 3,     /*!< Trigger Sequence 3 data*/
    ADC_TRIGGER_SEQUENCE_ID_4       = 4,     /**< Trigger Sequence 4 data*/
    ADC_TRIGGER_SEQUENCE_ID_5       = 5,     /**< Trigger Sequence 5 data*/
    ADC_TRIGGER_SEQUENCE_ID_6       = 6,     /*!< Trigger Sequence 6 data*/
    ADC_TRIGGER_SEQUENCE_ID_7       = 7,     /*!< Trigger Sequence 7 data*/
};

enum adc_trigger_timer_id
{
    ADC_TRIGGER_TIMER_ID_0  = 0,
    ADC_TRIGGER_TIMER_ID_1  = 1,
    ADC_TRIGGER_TIMER_ID_2  = 2,
    ADC_TRIGGER_TIMER_ID_3  = 3,
    ADC_TRIGGER_TIMER_ID_4  = 4,
    ADC_TRIGGER_TIMER_ID_5  = 5,
    ADC_TRIGGER_TIMER_ID_6  = 6,
    ADC_TRIGGER_TIMER_ID_7  = 7,
    ADC_TRIGGER_TIMER_ID_8  = 8,
    ADC_TRIGGER_TIMER_ID_9  = 9,
    ADC_TRIGGER_ASTART = 15,
};

/**
 * PFCP, MPWM total trigger source table. MPWM can't use ADC_TRIGGER_PWM_ID_BOTTOM_B and ADC_TRIGGER_PWM_ID_PERIOD_B
 */

enum adc_trigger_pwm_id
{
    ADC_TRIGGER_PWM_ID_ATR1  = 0,
    ADC_TRIGGER_PWM_ID_ATR2  = 1,
    ADC_TRIGGER_PWM_ID_ATR3  = 2,
    ADC_TRIGGER_PWM_ID_ATR4  = 3,
    ADC_TRIGGER_PWM_ID_ATR5  = 4,
    ADC_TRIGGER_PWM_ID_ATR6  = 5,
    ADC_TRIGGER_PWM_ID_BOTTOM_A = 6,
    ADC_TRIGGER_PWM_ID_PERIOD_A = 7,
    ADC_TRIGGER_PWM_ID_BOTTOM_B = 8,
    ADC_TRIGGER_PWM_ID_PERIOD_B = 9,
    ADC_TRIGGER_PWM_ASTART = 15,
};

/**
 * ADC external clock configuration
 */
enum adc_ext_clk
{
    ADC_INTERNAL_CLK,                       /**< Internal clock (CLKDIV enablement)*/

};

/**
 * ADC IO type
 */
enum adc_io_type
{
    ADC_IO_TYPE_BLOCK      = 0,             /**< blocking (Polling)*/
    ADC_IO_TYPE_NON_BLOCK  = 1,             /**< non-blocking (Interrupt, DMA) */
};

/**
 * ADC interrupt type
 */
enum adc_irq_type
{
    ADC_IRQ_SINGLE,                         /**< ADC single conversion interrupt*/
    ADC_IRQ_SEQUENCE,                  /**< ADC sequence conversion interrupt*/
    ADC_IRQ_TRIGGER,                     /**< ADC trigger conversion interrupt */
    ADC_IRQ_DMA,                            /**< ADC DMA done interrupt. */
    ADC_IRQ_INJECT,                        /**< ADC Inject Conversion end interrupt */
};

/**
 * ADC event
 */
enum adc_event
{
    ADC_EVENT_SINGLE    = BIT(0),           /**< ADC single conversion event*/
    ADC_EVENT_SEQUENCE  = BIT(1),           /**< ADC sequence conversion event*/
    ADC_EVENT_TRIGGER   = BIT(2),           /**< ADC trigger conversion event */
    ADC_EVENT_DMA       = BIT(3),           /**< ADC DMA done event */
    ADC_EVENT_INJECT   = BIT(4),            /**< ADC Inject converion end event */
};

/**
 * ADC sampling time of conversion sequence channel
 */
enum adc_conversion_sequence_id
{
    ADC_CONVERSION_SEQUENCE_0,                      /**< Conversion sequence 0 */
    ADC_CONVERSION_SEQUENCE_1,                      /**< Conversion sequence 1 */
    ADC_CONVERSION_SEQUENCE_2,                      /**< Conversion sequence 2 */
    ADC_CONVERSION_SEQUENCE_3,                      /**< Conversion sequence 3 */
    ADC_CONVERSION_SEQUENCE_4,                      /**< Conversion Sequence 4 */
    ADC_CONVERSION_SEQUENCE_5,                      /**< Conversion sequence 5 */
    ADC_CONVERSION_SEQUENCE_6,                      /**< Conversion sequence 6 */
    ADC_CONVERSION_SEQUENCE_7,                      /**< Conversion sequence 7 */
    ADC_CONVERSION_SEQUENCE_8,                      /**< Conversion sequence 8 */
    ADC_CONVERSION_SEQUENCE_9,                      /**< Conversion sequence 9 */
    ADC_CONVERSION_SEQUENCE_10,                     /**< Conversion sequence 10 */
    ADC_CONVERSION_SEQUENCE_11,                     /**< Conversion sequence 11 */
    ADC_CONVERSION_SEQUENCE_12,                     /**< Conversion sequence 12 */
    ADC_CONVERSION_SEQUENCE_13,                     /**< Conversion sequence 13 */
    ADC_CONVERSION_SEQUENCE_14,                     /**< Conversion Sequence 14 */
    ADC_CONVERSION_SEQUENCE_15,                     /**< Conversion sequence 15 */
    ADC_CONVERSION_SEQUENCE_16,                     /**< Conversion sequence 16 */
    ADC_CONVERSION_SEQUENCE_17,                     /**< Conversion sequence 17 */
    ADC_CONVERSION_SEQUENCE_18,                     /**< Conversion sequence 18 */
    ADC_CONVERSION_SEQUENCE_19,                     /**< Conversion sequence 19 */
    ADC_CONVERSION_SEQUENCE_20,                     /**< Conversion sequence 20 */
    ADC_CONVERSION_SEQUENCE_21,                     /**< Conversion sequence 21 */
    ADC_CONVERSION_SEQUENCE_22,                     /**< Conversion sequence 22 */
    ADC_CONVERSION_SEQUENCE_23,                     /**< Conversion sequence 23 */
    ADC_CONVERSION_SEQUENCE_MAX,
};

/**
 * ADC trigger type 
 */
enum adc_trigger_type
{
    ADC_TRIGGER_DISABLE,          /**< Event Trigger Disabled/Soft-Trigger Only */
    ADC_TRIGGER_TIMER,            /**< Timer Event Trigger */
    ADC_TRIGGER_MPWM0,            /**< MPWM0 Event Trigger */
    ADC_TRIGGER_MPWM1,            /**< MPWM1 Event Trigger */
    ADC_TRIGGER_PFCP,             /**< PFCP Event Trigger */
};

/**
 * ADC continuous type 
 */
enum adc_continous_type
{
    ADC_CONTINUOUS_NONE,
    ADC_CONTINUOUS_ADC0_ADC1,
    ADC_CONTINUOUS_ADC1_ADC0,
};

/**
 * ADC inject configuration
 */
struct adc_inject_cfg
{
    uint8_t trg_src;
    uint32_t sampling_time;
    uint8_t ch_sel;
    uint8_t trg_sel;
    uint8_t ext_start;
};

/**
 * ADC mode
 */
enum adc_input_mode
{
    ADC_INPUT_DIFFERENTIAL,                     /**< adc differential mode*/
    ADC_INPUT_SINGLE_END,                       /**< adc dingle-end mode*/
};

/**
 * ADC configuration
 */
struct adc_cfg
{
    enum adc_mode           mode;           /**< ADC_MODE_SINGLE, ADC_MODE_MULTI ,ADC_MODE_BURST */
    uint32_t                dma_opt;        /**< DMA option = ENABLE or DISABLE */
    uint32_t                sampling_time0;  /**< sampleing time */
    uint32_t                sampling_time1;  /**< sampleing time */
    uint32_t                sampling_time2;  /**< sampleing time */
    uint32_t                sampling_time3;  /**< sampleing time */
    uint32_t                sampling_time4;  /**< sampleing time */
    uint32_t                sampling_time5;  /**< sampleing time */
    uint32_t                sampling_time6;  /**< sampleing time */
    uint32_t                sampling_time7;  /**< sampleing time */
    uint32_t                seq_cnt;        /**< sequence count = count value 2~8  */
    uint32_t                restart_en;     /**< RestartEn = ENABLE or DISABLE */
    enum adc_ext_clk        use_clk;        /**< Internal clock, External clock (MCCR clock) */
    uint32_t                in_clk_div;     /**< clock divider */
    uint8_t                 irq_prio;       /**< interrupt priority */
    enum adc_ext_request    request_en;     /**< External Request = ENABLE or DISABLE */
    uint32_t                allstart_en;              /**< ADC all START bit */
};

/**
 *******************************************************************************
 * @brief       ADC interrupt callback function type
 * @param[in]   event event type
 * @param[in]   context context provided during initialization
 ******************************************************************************/
typedef void (*adc_irq_handler_t)(enum adc_id id, uint32_t event, void *context);

/**
 *******************************************************************************
 * @brief       Initialize ADC.
 * @param[in]   id instance id
 * @param[in]   cfg configuration
 * @param[in]   handler callback function to receive event
 * @param[in]   context caller's context
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_ADC_Init(enum adc_id id, struct adc_cfg *cfg,
                     adc_irq_handler_t handler,
                     void *context);

/**
 *******************************************************************************
 * @brief       Deintialize ADC.
 * @param[in]   id instance id
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_ADC_DeInit(enum adc_id id);

/**
 *******************************************************************************
 * @brief       Calibration ADC.
 * @param[in]   none
 * @return      none
 ******************************************************************************/
void HAL_ADC_Calibration(void);

/**
 *******************************************************************************
 * @brief       Set ADC clock.
 * @param[in]   id instance id
 * @param[in]   clk_div clock divider (0x01~0xFF)
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_ADC_SetClock(enum adc_id id, uint8_t clk_div);

/**
 *******************************************************************************
 * @brief       Set MR ADC.
 * @param[in]   id instance id
 * @param[in]   mode mode
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_ADC_SetMR(enum adc_id id, uint32_t mode);

/**
 *******************************************************************************
 * @brief       Get ADC data.
 * @param[in]   id instance id
 * @param[out]  data data
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_ADC_GetData(enum adc_id id, uint16_t *data);

/**
 *******************************************************************************
 * @brief       DMA control for ADC.
 * @param[in]   id instance id
 * @param[in]   ch dma channel
 * @param[in]   enable 1 to enable, 0 to disable
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_ADC_SetDMA(enum adc_id id, enum dma_ch ch, uint8_t enable);

/**
 *******************************************************************************
 * @brief       Get ADC data sequence.
 * @param[in]   id instance id
 * @param[in]   reg sequence  id
 * @param[out]  data data
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_ADC_GetSequenceData(enum adc_id id, enum adc_sequence_id reg, uint16_t *data);

/**
 *******************************************************************************
 * @brief       Start ADC single mode
 * @param[in]   id instance id
 * @param[out]   data pointer to the data
 * @param[in]   io_type operation type
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_ADC_StartSingle(enum adc_id id, uint32_t *data, enum adc_io_type io_type);

/**
 *******************************************************************************
 * @brief       Start ADC Burst mode
 * @param[in]   id instance id
 * @param[out]  data pointer to the data
 * @param[in]   io_type operation type
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_ADC_StartBurst(enum adc_id id, uint32_t *data, enum adc_io_type io_type);

/**
 *******************************************************************************
 * @brief       Start ADC sequence mode
 * @param[in]   id instance id
 * @param[out]  data pointer to the data
 * @param[in]   io_type operation type
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_ADC_StartSequence(enum adc_id id, uint32_t *data, enum adc_io_type io_type);

/**
 *******************************************************************************
 * @brief       Start ADC multiple mode
 * @param[in]   id instance id
 * @param[out]  data pointer to the data
 * @param[in]   io_type operation type
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_ADC_StartMultiple(enum adc_id id, uint32_t *data, enum adc_io_type io_type);

/**
 *******************************************************************************
 * @brief       Stop ADC
 * @param[in]   id instance id
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_ADC_Stop(enum adc_id id);

/**
 *******************************************************************************
 * @brief       Set AD conversion in power mode
 * @param[in]   id instance id
 * @param[in]   enable 1 to enable, 0 to disable
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_ADC_EnterPowerdownMode(enum adc_id id, uint8_t enable);

/**
 *******************************************************************************
 * @brief       Select ADC channel number
 * @param[in]   id instance id
 * @param[in]   seq id seuence ID
 * @param[in]   channel channel number
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_ADC_SelectChannel(enum adc_id id, enum adc_sequence_id seq_id, uint32_t channel);

/**
 *******************************************************************************
 * @brief       Select ADC channel number
 * @param[in]   id instance id
 * @param[in]   channel channel number
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_ADC_SelectChannel1(enum adc_id id, uint32_t channel);

/**
 *******************************************************************************
 * @brief       Select ADC channel number
 * @param[in]   id instance id
 * @param[in]   channel channel number
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_ADC_SelectChannel2(enum adc_id id, uint32_t channel);

/**
 *******************************************************************************
 * @brief       Select timer trigger
 * @param[in]   id instance id
 * @param[in]   trg_seq_id trigger sequecen id
 * @param[in]   trg_tm_id trigger timer id
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_ADC_SelectTriggerTimer(enum adc_id id, enum adc_trigger_sequence_id trg_seq_id,
                                   enum adc_trigger_timer_id trg_tm_id);

/**
 *******************************************************************************
 * @brief       Select timer or MPWM or PFC trigger
 * @param[in]   id instance id(ADC unit number)
 * @param[in]   trg_seq_id trigger sequecen id(adc->MR.b[SEQCNT])
 * @param[in]   adc_trigger_type trigger peripheral(software,Timer, MWPM,PFCP)
 * @param[in]   adc_trigger_pwm_id trg_pwm_id (trigger source table number, (adc->TRG)
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_ADC_SelectTriggerPWM(enum adc_id id, enum adc_trigger_sequence_id trg_seq_id,
                                  enum adc_trigger_type trg_type,
                                  enum adc_trigger_pwm_id trg_pwm_id);

/**
 *******************************************************************************
 * @brief       Get ADC channel status
 * @param[in]   id instance id
 * @param[in]   status status
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_ADC_GetStatus(enum adc_id id, uint32_t *status);

/**
 *******************************************************************************
 * @brief       Clear ADC channel status
 * @param[in]   id instance id
 * @param[in]   status status
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_ADC_ClearStatus(enum adc_id id, uint32_t status);

/**
 *******************************************************************************
 * @brief       Set sampling time of conversion sequence channel
 * @param[in]   id instance id
 * @param[in]   conv_seq_id conversion sequence id
 * @param[in]   sampling_time sampling time (0 to 31)
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_ADC_SetConvSamplingTime(enum adc_id id,
                                    enum adc_conversion_sequence_id conv_seq_id,
                                    uint16_t sampling_time);

/**
 *******************************************************************************
 * @brief       Enable ADC interrupt
 * @param[in]   id instance id
 * @param[in]   type interrupt type
 * @param[in]   enable 1 to enable, 0 to disable
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_ADC_SetIRQ(enum adc_id id, enum adc_irq_type type, uint8_t enable);


/**
 *******************************************************************************
 * @brief       Set ADC inject mode 
 * @param[in]   id instance id
 * @param[in]   cfg configuration
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_ADC_Inject_Init(enum adc_id id, struct adc_inject_cfg *cfg);

/**
 *******************************************************************************
 * @brief       Start ADC inject 
 * @param[in]   id instance id
 * @param[in]   cfg configuration
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_ADC_Inject_Start(enum adc_id id);


#ifdef __cplusplus
}
#endif

#endif /* _a34m456_ADC_H_ */

/** @} */
/** @} */

