/**
 *******************************************************************************
 * @file        a34m456_cmp.h
 * @author      ABOV R&D Division
 * @brief       12-bits Analog to Digital Converter
 *
 * Copyright 2024 ABOV Semiconductor Co.,Ltd. All rights reserved.
 *
 * This file is licensed under terms that are found in the LICENSE file
 * located at Document directory.
 * If this file is delivered or shared without applicable license terms,
 * the terms of the BSD-3-Clause license shall be applied.
 * Reference: https://opensource.org/licenses/BSD-3-Clause
 ******************************************************************************/

/**
 * @addtogroup  driver
 * @{
 * @defgroup    driver_cmp CMP
 * @{
 * @brief       COMPARATOR driver
 */

#ifndef _A34M456_CMP_H_
#define _A34M456_CMP_H_

#ifdef __cplusplus
extern "C" 
{
#endif

#include <stdint.h>
#include "a34m456_driver_common.h"
#include "a34m456_dma.h"

#define CMP_WRITE_BUSY_POLLING         while ((CMPn->STAT & CMP_STAT_CMPBUSY_Msk) == (0x01UL << CMP_STAT_CMPBUSY_Pos))

/**
 * CMP instance id
 */

enum cmp_id
{
    CMP_ID_0   = 0,                        /**< CMP 0 */
    CMP_ID_1   = 1,                        /**< CMP 1 */
    CMP_ID_2   = 2,                        /**< CMP 2 */
    CMP_ID_3   = 3,                        /**< CMP 3 */
    CMP_ID_MAX = 4,                        /**< CMP max */	
};

/**
 * COMP Filter divider
 */
enum cmp_div
{
    CMP_DIV_1,                          /**< Filter divider 1 */
    CMP_DIV_2,                          /**< Filter divider 2 */
    CMP_DIV_4,                          /**< Filter divider 4 */
    CMP_DIV_8,                          /**< Filter divider 8 */
    CMP_FILTER_MAX,                        /**< Filter count MAX - should not used this */
};

/**
 * CMP hystresis type
 */
enum cmp_hystresis_type
{
    CMP_HYSTERESIS_40MV  = 0,    /**< 40 mV */
    CMP_HYSTERESIS_80MV  = 1,    /**< 80 mV */
    CMP_HYSTERESIS_120MV = 2,    /**< 120 mV */
    CMP_HYSTERESIS_160MV = 3     /**< 160 mV */
};

/**
 * CMP polarity type
 */
typedef enum 
{
    CMP_EVENT_LOW_FALLING = 0x00UL, /**< event at low or falling */
    CMP_EVENT_HIGH_RISING = 0x01UL  /**< event at high or rising */
} CMP_EventPolarity_Type;

/**
 * CMP interrupt type
 */
enum cmp_irq_type
{
    CMP_IRQ_NONE,                          /**< interrupt disabled */
    CMP_IRQ_LEVEL,                         /**< interrupt on level */
    CMP_IRQ_LOW_EDGE,                      /**< interrupt on low (falling) edge */
    CMP_IRQ_HIGH_EDGE,                     /**< interrupt on high (rising) edge */
    CMP_IRQ_BOTH_EDGE,                     /**< interrupt on both edges */
};

/**
 * CMP input(- negative) reference  selection
 */
enum cmp_input_neg
{
    CMP_INPUT_REFERENCE_CPnB       = 0x00UL,                  /**< negative reference CMPnR */
    CMP_INPUT_REFERENCE_DACO0      = 0x01UL,                  /**< negative reference DACO0 */
    CMP_INPUT_REFERENCE_DACO1      = 0x02UL,                 /**< negative reference DACO1 */
    CMP_INPUT_REFERENCE_VREF       = 0x03UL,                   /**< negative reference Vref(1.0V) */
};

/**
 * CMP input reference positive selection
 */
enum cmp_input_pos
{
    CMP_INPUT_SOURCE_CPnA0         = 0x00UL,                   /**< Positive source CP0A0, CP1A0, CP2A0, CP3A0 */
    CMP_INPUT_SOURCE_01            = 0x01UL,                   /**< Positive source CP0A1, CP1A1, CP2A1, DACO0 */
    CMP_INPUT_SOURCE_10            = 0x02UL,                   /**< Positive source DACO0,DACO0,DACO1,DACO1 */
    CMP_INPUT_SOURCE_VREF          = 0x03UL,                   /**< Positive source Vref(1.0V) */
};
/**
 * CMP status
 */
enum cmp_status
{
    CMP_STATUS_DOWN,                       /**< comparator voltage crossed down */
    CMP_STATUS_UP,                         /**< comparator voltage crossed up */
};

/**
 * CMP event
 */
enum cmp_event
{
    CMP_EVENT_DOWN,                         /**< comparator voltage crossed down */
    CMP_EVENT_UP,                           /**< comparator voltage crossed up */

};

/**
 *******************************************************************************
 * @brief       CMP interrupt callback function type
 * @param[in]   event event type ::comp_event
 * @param[in]   context context provided during initialization
 ******************************************************************************/
typedef void (*cmp_irq_handler_t)(uint32_t event, void *context);

/**
 * CMP configuration
 */
struct cmp_cfg
{
    uint8_t             filter_count;     /**< filter count*/
    enum cmp_div           filter_div;     /**< filter divider*/
    enum cmp_hystresis_type    hystresis;  /**< hystresis */
    enum cmp_irq_type          irq_type;   /**< interrupt type */
    enum cmp_input_neg         input_neg;  /**< negative input */
    enum cmp_input_pos         input_pos;  /**< positive input */
    uint8_t                     irq_prio;   /**< interrupt priority */
};
/**
 *******************************************************************************
 * @brief       Initializes the Comparator peripheral according to the specified  
 *              parameters in the pConfig and pConfig2.
 * @param[in]   id : CMP channel selection
 *               - CMP0 : CMP0 peripheral
 *               - CMP1 : CMP1 peripheral
 *               - CMP2 : CMP2 peripheral
 *               - CMP3 : CMP3 peripheral
 * @param[in]   pConfig : Pointer to a CMP_CFG_Type structure  
 *              that contains the configuration information for the  
 *              specified CMP peripheral.
 * @param[in]   pConfig2 : Pointer to a CMP_CR_Type structure  
 *              that contains the Interrupt information for the  
 *              specified CMP peripheral.
 * @return      void : None
 ******************************************************************************/
int32_t HAL_CMP_Init(enum cmp_id id, struct cmp_cfg *cfg,
                      cmp_irq_handler_t handler,
                      void *context);

/**
 *******************************************************************************
 * @brief       De-initializes the CMP peripheral registers to their  
 *              default reset values.
 * @param[in]  void : None
 * @return     void : None
 ******************************************************************************/
int32_t HAL_CMP_Deinit(enum cmp_id id);

/**
 *******************************************************************************
 * @brief       Start Comparatar
 * @param[in]   CMP : CMP peripheral selected, should be
 *               - CMP0 : CMP0 peripheral
 *               - CMP1 : CMP1 peripheral
 *               - CMP2 : CMP2 peripheral
 *               - CMP3 : CMP3 peripheral
 * @return      void : None
 ******************************************************************************/
int32_t HAL_CMP_Start(enum cmp_id id);

/**
 *******************************************************************************
 * @brief       Stop Comparatar
 * @param[in]   CMP : CMP peripheral selected, should be
 *               - CMP0 : CMP0 peripheral
 *               - CMP1 : CMP1 peripheral
 *               - CMP2 : CMP2 peripheral
 *               - CMP3 : CMP3 peripheral
 * @return      void : None
 ******************************************************************************/
int32_t HAL_CMP_Stop(enum cmp_id id);

/**
 *******************************************************************************
 * @brief       Get CMP channel status
 * @param[in]   ids : CMP peripheral selected, should be
 *               - CMPn : 0~3 CMP peripheral
 * @return      CMP status register
 ******************************************************************************/
int32_t HAL_CMP_GetStatus(enum cmp_id id, uint8_t *status);



#ifdef __cplusplus
}
#endif


#endif /* _A34M456_ADC_H_ */

/** @} */
/** @} */
