/**
 *******************************************************************************
 * @file        a34m456_crc.h
 * @author      ABOV R&D Division
 * @brief       crc driver
 *
 * Copyright 2024 ABOV Semiconductor Co.,Ltd. All rights reserved.
 *
 * This file is licensed under terms that are found in the LICENSE file
 * located at Document directory.
 * If this file is delivered or shared without applicable license terms,
 * the terms of the BSD-3-Clause license shall be applied.
 * Reference: https://opensource.org/licenses/BSD-3-Clause
 ******************************************************************************/

/**
 * @addtogroup  driver
 * @{
 * @defgroup    driver_crc CRC
 * @{
 * @brief       CRC driver
 */


#ifndef _A34M456_CRC_H_
#define _A34M456_CRC_H_

#ifdef __cplusplus
extern "C"
{
#endif

#include <stdint.h>
#include "a34m456_driver_common.h"
#include "a34m456_dma.h"

/**
 * CRC Polinomial type
 */
enum crc_poly
{
    CRC_POLY_CRC32               = 0,       /**< CRC32 mode */
    CRC_POLY_CRC16_USB           = 1,       /**< CRC16_USB mode */
    CRC_POLY_CRC16_CCITT         = 2,       /**< CRC16_CCITT mode */
    CRC_POLY_CRC8                = 3,       /**< CRC8 mode */
};

/**
 * CRC IO type
 */
enum crc_io_type
{
    CRC_IO_TYPE_BLOCK       = 0,            /**< blocking */
    CRC_IO_TYPE_NON_BLOCK   = 1,            /**< non-blocking */
};

/**
 * CRC event
 */
enum crc_event
{
    CRC_EVENT_DONE  = 0,                    /**< generation complete */
    CRC_EVENT_ERROR = 1,                    /**< generation error */
};

/**
 *******************************************************************************
 * @brief       CRC interrupt callback function type
 * @param[in]   event event type ::crc_event
 * @param[in]   context context provided during initialization
 ******************************************************************************/
typedef void (*crc_irq_handler_t)(uint32_t event, void *context);

/**
 * CRC configuration
 */
struct crc_cfg
{
    uint32_t    out_inv;            /**< output data bit is inverted */
    uint32_t    out_rev;            /**< oupput data reverse */
    uint32_t    in_rev;             /**< input data reverse */
    uint32_t    poly;               /**< polinomial */
    uint8_t     irq_prio;           /**< interrupt priority */
	uint32_t    init;				/**< init data */
};


/**
 *******************************************************************************
 * @brief       Initialize CRC.
 * @param[in]   cfg configuration
 * @param[in]   handler callback function to receive event
 * @param[in]   context caller's context
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_CRC_Init(struct crc_cfg *cfg, crc_irq_handler_t handler, void *context);

/**
 *******************************************************************************
 * @brief       Deinitialize CRC.
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_CRC_Deinit(void);

/**
 *******************************************************************************
 * @brief       Enable DMA for CRC.
 * @param[in]   ch DMA channel
 * @param[in]   enable 1 to enable, 0 to disable
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_CRC_SetDMA(enum dma_ch ch, uint8_t enable);

/**
 *******************************************************************************
 * @brief       Compute CRC.
 *              If DMA is enabled before calling this function, the result
 *              value is available when DONE event is signalled.
 *              Otherwise, the function waits until the result is available.
 * @param[in]   init initial value
 * @param[in]   data address of the data
 * @param[in]   len length of the data
 * @param[out]  value value of the computed CRC
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_CRC_Compute(uint32_t init, uint8_t *data, uint32_t len, uint32_t *value);


#ifdef __cplusplus
}
#endif

#endif /* _A34M456_CRC_H_ */

/** @} */
/** @} */


