/**
 *******************************************************************************
 * @file        a34m456_mpwm.h
 * @author      ABOV R&D Division
 * @brief       mpwm driver
 *
 * Copyright 2024 ABOV Semiconductor Co.,Ltd. All rights reserved.
 *
 * This file is licensed under terms that are found in the LICENSE file
 * located at Document directory.
 * If this file is delivered or shared without applicable license terms,
 * the terms of the BSD-3-Clause license shall be applied.
 * Reference: https://opensource.org/licenses/BSD-3-Clause
 ******************************************************************************/

/**
 * @addtogroup  driver
 * @{
 * @defgroup    driver_mpwm MPWM
 * @{
 * @brief       MPWM driver
 */

#ifndef _A34M456_MPWM_H_
#define _A34M456_MPWM_H_

#ifdef __cplusplus
extern "C"
{
#endif

#include <stdint.h>
#include "a34m456_driver_common.h"

/**
 * MPWM instance id
 */
enum mpwm_id
{
    MPWM_ID_0   = 0,                        /**< MPWM 0 */
    MPWM_ID_1   = 1,                        /**< MPWM 1 */
    MPWM_ID_MAX = 2,                        /**< MPWM max */
};

/**
 * MPWM mode
 */
enum mpwm_mode
{
    MPWM_MODE_MOTOR0         = 0,            /**< MPWM moter mode0 */
    MPWM_MODE_MOTOR1         = 1,            /**< MPWM moter mode1 */
    MPWM_MODE_NORMAL         = 2,            /**< MPWM nomral mode */
};

/**
 * MPWM channel mode
 */
enum mpwm_chan_mode
{
    MPWM_CHAN_MODE_2CH_SYMMETRIC    = 0,    /**< Two-channel symmetric mode */
    MPWM_CHAN_MODE_1CH_ASYMMETRIC   = 1,    /**< One-channel asymmetric mode */
    MPWM_CHAN_MODE_1CH_SYMMETRIC    = 2,    /**< One-channel symmetric mode */
};


/**
 * MPWM phase type
 */
enum mpwm_phase_type
{
    MPWM_PHASE_TYPE_U   = 0,                /**< MPWM phase U */
    MPWM_PHASE_TYPE_V   = 1,                /**< MPWM phase V */
    MPWM_PHASE_TYPE_W   = 2,                /**< MPWM phase W */
    MPWM_PHASE_TYPE_ALL = 3,                /**< MPWM phase ALL */
};

/**
 * MPWM output type
 */
enum mpwm_output_type
{
    MPWM_OUTPUT_TYPE_L  = 0,                /**< gate low side*/
    MPWM_OUTPUT_TYPE_H  = 1,                /**< gate high side */
};

/**
 * MPWM output level update setting 
 */
enum mpwm_output_lv_update
{
    MPWM_OUTPUT_LEVEL_SPECIFIED_TIME        = 0,        /**< Updates at specified timings */
    MPWM_OUTPUT_LEVEL_AT_ONCE               = 1,        /**< Updates all output levels at once */
};
/**
 * MPWM output level update top(period) setting 
 */
enum mpwm_output_lv_update_top
{
    MPWM_OUTPUT_LEVEL_TOP_NOT_UPDATE        = 0,        /**< MPWM_OLR register is not updated at every period-match. */
    MPWM_OUTPUT_LEVEL_TOP_UPDATE            = 1,        /**< MPWM_OLR register is updated at every period-match. */
};
/**
 * MPWM output level update bottom setting 
 */
enum mpwm_output_lv_update_btm
{
    MPWM_OUTPUT_LEVEL_BTM_NOT_UPDATE        = 0,        /**< MPWM_OLR register is not updated at every bottom-match. */
    MPWM_OUTPUT_LEVEL_BTM_UPDATE            = 1,        /**< MPWM_OLR register is updated at every bottom-match. */
};
/**
 * MPWM period&duty update setting 
 */
enum mpwm_prd_duty_update
{
    MPWM_PRD_DUTY_SPECIFIED_TIME        = 0,        /**< Updates duty and period at specified timings */
    MPWM_PRD_DUTY_AT_ONCE               = 1,        /**< Updates duty and period at once */
};
/**
 * MPWM period&duty update top(period) setting 
 */
enum mpwm_prd_duty_update_top
{
    MPWM_PRD_DUTY_TOP_NOT_UPDATE        = 0,        /**< Period and Duty value are not updated at every period-match. */
    MPWM_PRD_DUTY_TOP_UPDATE            = 1,        /**< Period and Duty value are updated at every period-match. */
};
/**
 * MPWM period&duty update bottom setting 
 */
enum mpwm_prd_duty_update_btm
{
    MPWM_PRD_DUTY_BTM_NOT_UPDATE        = 0,        /**< Period and Duty value are not updated at every bottom-match.. */
    MPWM_PRD_DUTY_BTM_UPDATE            = 1,        /**< Period and Duty value are updated at every bottom-match. */
};
/**
 * MPWM dead time mode
 */
enum mpwm_dead_time_mode
{
    MPWM_DEAD_TIME_MODE_LH_TL   = 0,        /**< Inserts dead time at the leading edge of MPWMxH and the trailing edge of MPWMxL */
    MPWM_DEAD_TIME_MODE_TH_LL   = 1,        /**< Inserts dead time at the trailing edge of MPWMxH and the leading edge of MPWMxL */
};


/**
 * MPWM trigger souce of ADC
 */
enum mpwm_adc_trigger_source
{
    MPWM_ADC_TRIGGER_SOURCE_U_COUNTER   = 0,    /**< Set MPWM phase U to trigger of ADC */
    MPWM_ADC_TRIGGER_SOURCE_V_COUNTER   = 1,    /**< Set MPWM phase V to trigger of ADC */
    MPWM_ADC_TRIGGER_SOURCE_W_COUNTER   = 2,    /**< Set MPWM phase W to trigger of ADC */
};
/**
 * MPWM trigger register update mode
 */
enum mpwm_adc_trigger_update
{
    MPWM_ADC_TRIGGER_UPDATE_PERIOD      = 0,    /**< ADC trigger value applied at period match event */
    MPWM_ADC_TRIGGER_UPDATE_2CLOCK      = 1,    /**< written trigger register values are sent to the trigger compare block after two PWM clock cycles */
};

/**
 * MPWM trigger mode of ADC
 */
enum mpwm_adc_trigger_mode
{
    MPWM_ADC_TRIGGER_MODE_DISABLE       = 0,    /**< Disable trigger mode */
    MPWM_ADC_TRIGGER_MODE_UP_COUNT      = 1,    /**< Set up count match trigger mode */
    MPWM_ADC_TRIGGER_MODE_DOWN_COUNT    = 2,    /**< Set down count match trigger mode */
    MPWM_ADC_TRIGGER_MODE_UPDOWN_COUNT  = 3,    /**< Set up-down count match trigger mode */
};

/**
 * MPWM trigger id of ADC
 */
enum mpwm_trgger_id
{
    MPWM_TRGGER_ID_1    = 0,            /**< MPWM Trigger 1 */
    MPWM_TRGGER_ID_2    = 1,            /**< MPWM Trigger 2 */
    MPWM_TRGGER_ID_3    = 2,            /**< MPWM Trigger 3 */
    MPWM_TRGGER_ID_4    = 3,            /**< MPWM Trigger 4 */
    MPWM_TRGGER_ID_5    = 4,            /**< MPWM Trigger 5 */
    MPWM_TRGGER_ID_6    = 5,            /**< MPWM Trigger 6 */
    MPWM_TRGGER_ID_MAX  = 6,            /**< MPWM Trigger max */
};

/**
 * MPWM interrupt event type
 */
enum mpwm_irq_event_type
{
    MPWM_IRQ_EVENT_TYPE_PRT = 1,    /**< IRQ event protection */
    MPWM_IRQ_EVENT_TYPE_OC = 2,    /**< IRQ event over current */
    MPWM_IRQ_EVENT_TYPE_MPWM = 4,    /**< IRQ event MPWM(bottom/period/duty/ATR) */
};


/**
 * MPWM interrupt period
 */
enum mpwm_prdie
{
    MPWM_PRDIE_DISABLE     = 0,
    MPWM_PRDIE_ENABLE      = 1,
};
/**
 * MPWM interrupt bottom
 */
enum mpwm_botie
{
    MPWM_BOTIE_DISABLE     = 0,
    MPWM_BOTIE_ENABLE      = 1,
};
/**
 * MPWM interrupt atr6ie and WH ie
 */
enum mpwm_atr6ie
{
    MPWM_ATR6IE_DISABLE     = 0,
    MPWM_ATR6IE_ENABLE      = 1,
};
/**
 * MPWM interrupt atr5ie and VH ie
 */
enum mpwm_atr5ie
{
    MPWM_ATR5IE_DISABLE     = 0,
    MPWM_ATR5IE_ENABLE      = 1,
};
/**
 * MPWM interrupt atr4ie and UH ie
 */
enum mpwm_atr4ie
{
    MPWM_ATR4IE_DISABLE     = 0,
    MPWM_ATR4IE_ENABLE      = 1,
};
/**
 * MPWM interrupt atr3ie and WL ie
 */
enum mpwm_atr3ie
{
    MPWM_ATR3IE_DISABLE     = 0,
    MPWM_ATR3IE_ENABLE      = 1,
};
/**
 * MPWM interrupt atr2ie and VL ie
 */
enum mpwm_atr2ie
{
    MPWM_ATR2IE_DISABLE     = 0,
    MPWM_ATR2IE_ENABLE      = 1,
};
/**
 * MPWM interrupt atr2ie and UL ie
 */
enum mpwm_atr1ie
{
    MPWM_ATR1IE_DISABLE     = 0,
    MPWM_ATR1IE_ENABLE      = 1,
};
/**
 * MPWM Dead time function enable
 */
enum mpwm_dten
{
    MPWM_DT_DISABLE     = 0,
    MPWM_DT_ENABLE      = 1,
};
/**
 * MPWM PSHRT enable
 */
enum mpwm_pshrt
{
    MPWM_PSHRT_DISABLE     = 0,
    MPWM_PSHRT_ENABLE      = 1,
};
/**
 * MPWM HDTOL. output level at high side dead time 
 */
enum mpwm_hdtol
{
    MPWM_HDTOL_LOW_LEVEL     = 0,
    MPWM_HDTOL_HIGH_LEVEL    = 1,
};
/**
 * MPWM LDTOL. output level at low side dead time 
 */
enum mpwm_ldtol
{
    MPWM_LDTOL_LOW_LEVEL     = 0,
    MPWM_LDTOL_HIGH_LEVEL    = 1,
};
/**
 * MPWM dead clock div
 */
enum mpwm_dead_time_clk
{
    MPWM_DEAD_TIME_CLK_PWM_DIV_2    = 0,    /**< MPWM clock devide 2 */
    MPWM_DEAD_TIME_CLK_PWM_DIV_4    = 1,    /**< MPWM clock devide 4 */
    MPWM_DEAD_TIME_CLK_PWM_DIV_8    = 2,    /**< MPWM clock devide 8 */
    MPWM_DEAD_TIME_CLK_PWM_DIV_16   = 3,    /**< MPWM clock devide 16 */
};

/**
 * MPWM protection & OC config polarity
 */
enum mpwm_prt_oc_polarity
{
    MPWM_PRT_OC_POLARITY_LOW        = 0,    /**< L active */
    MPWM_PRT_OC_POLARITY_HIGH       = 1,    /**< H active */
};
/**
 * MPWM protection
 */
enum mpwm_enable
{
    MPWM_PRT_DISABLE       = 0,    /**< Disable */
    MPWM_PRT_ENABLE        = 1,    /**< Enable */
};
/**
 * MPWM  OC
 */
enum mpwm_oc_enable
{
    MPWM_OC_DISABLE       = 0,    /**< Disable */
    MPWM_OC_ENABLE        = 1,    /**< Enable */
};
/**
 *******************************************************************************
 * @brief       MWPM interrupt callback function type
 * @param[in]   event event type (::mpwm_irq_event_type)
 * @param[in]   context context provided during initialization
 ******************************************************************************/
typedef void (*mpwm_irq_handler_t)(uint32_t evnet, void *context);

/**
 * MPWM interrupt configuration
 */
struct mpwm_irq_cfg
{
    enum mpwm_prdie          prdie;             /**< MPWM Phase U, V, and W channels period interrupt */
    enum mpwm_botie          botie;             /**< MPWM Phase U, V, and W channels bottom interrupt */
    enum mpwm_atr6ie         atr6ie;            /**< WH Duty or ATR6 match interrupt */
    enum mpwm_atr5ie         atr5ie;            /**< VH Duty or ATR5 match interrupt */
    enum mpwm_atr4ie         atr4ie;            /**< UH Duty or ATR4 match interrupt */
    enum mpwm_atr3ie         atr3ie;            /**< WL Duty or ATR3 match interrupt */
    enum mpwm_atr2ie         atr2ie;            /**< VL Duty or ATR2 match interrupt */
    enum mpwm_atr1ie         atr1ie;            /**< UL Duty or ATR1 match interrupt */
    uint8_t                  irq_prio;                   /**< interrupt priority */
};

/**
 * MPWM Mode register configuration
 */
struct mpwm_cfg
{
    enum mpwm_mode                      mode;           /**< MPWM mode */
    enum mpwm_chan_mode                 chan_mode;      /**< MPWM channel mode */
    enum mpwm_output_lv_update          update_lv_ao;   /**< MPWM Output level update at once */
    enum mpwm_output_lv_update_top      top_upd_lv;     /**< MPWM Top update output level */
    enum mpwm_output_lv_update_btm      btm_upd_lv;     /**< MPWM Bottom update output level */
    enum mpwm_prd_duty_update           update_ao;      /**< MPWM duty, period update at once */
    enum mpwm_prd_duty_update_top       top_upd_prd;    /**< MPWM Top update period(top) and duty */
    enum mpwm_prd_duty_update_btm       btm_upd_prd;    /**< MPWM Bottom update output level */
    uint32_t clk_div;                                   /**< 4bit clock divider */
};

/**
 * MPWM input protection configuration
 */
struct mpwm_input_prt_cfg
{
    enum mpwm_prt_oc_polarity     polarity;              /**< input polarity (0:L-active, 1:H-active) */
    enum mpwm_enable           enable_prt7;       /**< enable or disable ADC2 comparator 0 protection  */
    enum mpwm_enable           enable_prt6;       /**< enable or disable ADC1 comparator 0 protection  */
    enum mpwm_enable           enable_prt5;       /**< enable or disable ADC0 comparator 0 protection  */
    enum mpwm_enable           enable_prt4;       /**< enable or disable CMP3 protection  */
    enum mpwm_enable           enable_prt3;       /**< enable or disable CMP2 protection  */
    enum mpwm_enable           enable_prt2;       /**< enable or disable CMP1 protection  */
    enum mpwm_enable           enable_prt1;       /**< enable or disable CMP0 protection  */
    enum mpwm_enable           enable_prt0;       /**< enable or disable PRTIN or OCPIN protection  */
    uint32_t clk_div;                                    /**< 3bit debounce clock divider */
    uint32_t debounce;                                   /**< debounce time (0:no debounce, 0x01 ~ 0xFF:MPWM clock * debounce, ) */
};

/**
 * MPWM input over current configuration
 */
struct mpwm_input_oc_cfg
{
    enum mpwm_prt_oc_polarity     polarity;              /**< input polarity (0:L-active, 1:H-active) */
    enum mpwm_oc_enable           enable_prt7;       /**< enable or disable ADC2 comparator 0 protection  */
    enum mpwm_oc_enable           enable_prt6;       /**< enable or disable ADC1 comparator 0 protection  */
    enum mpwm_oc_enable           enable_prt5;       /**< enable or disable ADC0 comparator 0 protection  */
    enum mpwm_oc_enable           enable_prt4;       /**< enable or disable CMP3 protection  */
    enum mpwm_oc_enable           enable_prt3;       /**< enable or disable CMP2 protection  */
    enum mpwm_oc_enable           enable_prt2;       /**< enable or disable CMP1 protection  */
    enum mpwm_oc_enable           enable_prt1;       /**< enable or disable CMP0 protection  */
    enum mpwm_oc_enable           enable_prt0;       /**< enable or disable PRTIN or OCPIN protection  */
    uint32_t clk_div;                                    /**< 3bit debounce clock divider */
    uint32_t debounce;                                   /**< debounce time (0:no debounce, 0x01 ~ 0xFF:MPWM clock * debounce, ) */
};

/**
 * MPWM deatime configuration
 */
struct mpwm_dt_cfg
{
    enum mpwm_dten              enable;                     /**< enable or disable dead time function */
    enum mpwm_pshrt             short_circuit_protect_en;   /**< enable of disable short circuit protection */
    enum mpwm_hdtol             ouput_level_high_side;      /**< dead time output level of high side */
    enum mpwm_ldtol             ouput_level_low_side;       /**< dead time output level of low side */
    enum mpwm_dead_time_mode    mode;                       /**< unused */
    enum mpwm_dead_time_clk     clk;                        /**< prescalter of dead time */
    uint32_t                    hside_dead_time;            /**< high side dead time value */
    uint32_t                    lside_dead_time;            /**< low side dead time value */
};

/**
 * MPWM ADC trigger configuration
 */
struct mpwm_adc_tr_cfg
{
    uint32_t                        reg_mode;       /**< ADC register update mode */
    enum mpwm_adc_trigger_mode      mode;           /**< ADC trigger mode */
    uint32_t                        tr_count;       /**< ADC trigger count value */
};

/**
 *******************************************************************************
 * @brief       Initialize MPWM.
 * @param[in]   id instance id
 * @param[in]   cfg configuration
 * @param[in]   handler callback function to receive event
 * @param[in]   context caller's context
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_MPWM_Init(enum mpwm_id id, struct mpwm_cfg *cfg,
                      mpwm_irq_handler_t handler,
                      void *context);

/**
 *******************************************************************************
 * @brief       Deintialize MPWM.
 * @param[in]   id instance id
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_MPWM_Deinit(enum mpwm_id id);

/**
 *******************************************************************************
 * @brief       configure MPWM interrupt type.
 * @param[in]   id instance id
 * @param[in]   cfg cofigureation
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_MPWM_SetIRQConfig(enum mpwm_id id, struct mpwm_irq_cfg *cfg);

/**
 *******************************************************************************
 * @brief       configure MPWM input protection.
                It is possible that set each phase protection in individual mode.
 * @param[in]   id instance id
 * @param[in]   handler call back lable
 * @param[in]   cfg cofigureation
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_MPWM_SetPotectionInputConfig(enum mpwm_id id, mpwm_irq_handler_t handler,
                                         struct mpwm_input_prt_cfg *cfg);

/**
 *******************************************************************************
 * @brief       configure MPWM output protection.
 * @param[in]   id instance id
 * @param[in]   handler call back lable
 * @param[in]   ot MPWM output type
 * @param[in]   en enable
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_MPWM_SetProtectionOutputConfig(enum mpwm_id id,mpwm_irq_handler_t handler,
                                           enum mpwm_output_type ot, uint32_t en);

/**
 *******************************************************************************
 * @brief       configure MPWM input over current.
                It is possible that set each phase over volatge in individual mode.
 * @param[in]   id instance id
 * @param[in]   handler call back lable
 * @param[in]   cfg cofigureation
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_MPWM_SetOverCurrentInputConfig(enum mpwm_id id, mpwm_irq_handler_t handler,
                                           struct mpwm_input_oc_cfg *cfg);

/**
 *******************************************************************************
 * @brief       configure MPWM output over current.
 * @param[in]   id instance id
 * @param[in]   handler call back lable
 * @param[in]   ot MPWM output type
 * @param[in]   en enable
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_MPWM_SetOverCurrentOutputConfig(enum mpwm_id id, mpwm_irq_handler_t handler,
                                            enum mpwm_output_type ot, uint32_t en);

/**
 *******************************************************************************
 * @brief       configure MPWM dead time.
                It is possible that set each phase dead time in individual mode.
 * @param[in]   id instance id
 * @param[in]   p_type MPWM phase type
 * @param[in]   cfg cofigureation
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_MPWM_SetDeadTimeConfig(enum mpwm_id id, enum mpwm_phase_type p_type, struct mpwm_dt_cfg *cfg);

/**
 *******************************************************************************
 * @brief       configure MPWM trigger of ADC.
 * @param[in]   id instance id
 * @param[in]   t_id MPWM trigger id
 * @param[in]   cfg cofigureation
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_MPWM_SetADCTriggerConfig(enum mpwm_id id, enum mpwm_trgger_id t_id, struct mpwm_adc_tr_cfg *cfg);

/**
 *******************************************************************************
 * @brief       set MPWM IRQ interval.
                It is possible that set each phase IRQ interval in individual mode.
 * @param[in]   id instance id
 * @param[in]   p_type MPWM phase type
 * @param[in]   irq_intv IRQ interval value
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_MPWM_SetIRQinterval(enum mpwm_id id, enum mpwm_phase_type p_type,
                                uint32_t irq_intv);

/**
 *******************************************************************************
 * @brief       control MPWM output.
 * @param[in]   id instance id
 * @param[in]   p_type MPWM phase type
 * @param[in]   ot MPWM output type
 * @param[in]   en enable(1) or disable(0)
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_MPWM_ControlPWMOutput(enum mpwm_id id,
                                        enum mpwm_phase_type p_type,
                                        enum mpwm_output_type ot, uint32_t en);

/**
 *******************************************************************************
 * @brief       set MPWM output level at disable phase.
 * @param[in]   id instance id
 * @param[in]   p_type MPWM phase type
 * @param[in]   ot MPWM output type
 * @param[in]   hl high(1) or low(0)
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_MPWM_SetDisabledOutputLevel(enum mpwm_id id,
                                        enum mpwm_phase_type p_type,
                                        enum mpwm_output_type ot, uint32_t hl);

/**
 *******************************************************************************
 * @brief       set MPWM start output level.
 * @param[in]   id instance id
 * @param[in]   p_type MPWM phase type
 * @param[in]   ot MPWM output type
 * @param[in]   hl high(1) or low(0)
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_MPWM_SetStartOutputLevel(enum mpwm_id id,
                                     enum mpwm_phase_type p_type,
                                     enum mpwm_output_type ot, uint32_t hl);

/**
 *******************************************************************************
 * @brief       set MPWM force output level.
 * @param[in]   id instance id
 * @param[in]   p_type MPWM phase type
 * @param[in]   ot MPWM output type
 * @param[in]   hl high(1) or low(0)
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_MPWM_SetForceOutputLevel(enum mpwm_id id,
                                     enum mpwm_phase_type p_type,
                                     enum mpwm_output_type ot, uint8_t hl);

/**
 *******************************************************************************
 * @brief       set MPWM period.
                It is possible that set each phase period in individual mode.
 * @param[in]   id instance id
 * @param[in]   p_type MPWM phase type
 * @param[in]   prd period value
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_MPWM_SetPeriod(enum mpwm_id id, enum mpwm_phase_type p_type, uint16_t prd);

/**
 *******************************************************************************
 * @brief       set MPWM duty.
 * @param[in]   id instance id
 * @param[in]   p_type MPWM phase type
 * @param[in]   ot MPWM output type
 * @param[in]   duty period value
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_MPWM_SetDuty(enum mpwm_id id, enum mpwm_phase_type p_type,
                         enum mpwm_output_type ot, uint16_t duty);

/**
 *******************************************************************************
 * @brief       start MPWM.
                It is possible that start each phase in individual mode.
 * @param[in]   id instance id
 * @param[in]   p_type MPWM phase type
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_MPWM_Start(enum mpwm_id id, enum mpwm_phase_type p_type);

/**
 *******************************************************************************
 * @brief       stop MPWM.
                It is possible that stop each phase in individual mode.
 * @param[in]   id instance id
 * @param[in]   p_type MPWM phase type
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_MPWM_Stop(enum mpwm_id id, enum mpwm_phase_type p_type);

/**
 *******************************************************************************
 * @brief       Halt MPWM.
                It is possible that halt each phase in individual mode.
 * @param[in]   id instance id
 * @param[in]   p_type MPWM phase type
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_MPWM_Halt(enum mpwm_id id, enum mpwm_phase_type p_type);

/**
 *******************************************************************************
 * @brief       Resume MPWM.
                It is possible that resume each phase in individual mode.
 * @param[in]   id instance id
 * @param[in]   p_type MPWM phase type
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_MPWM_Resume(enum mpwm_id id, enum mpwm_phase_type p_type);

/**
 *******************************************************************************
 * @brief       read count.
                It is possible that read count of each phase in individual mode.
 * @param[in]   id instance id
 * @param[in]   p_type MPWM phase type
 * @param[out]  count MPWM count
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_MPWM_ReadCount(enum mpwm_id id, enum mpwm_phase_type p_type, uint16_t *count);

/**
 *******************************************************************************
 * @brief       read status.
 * @param[in]   id instance id
 * @param[out]  status MPWM status
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_MPWM_ReadStatus(enum mpwm_id id, uint32_t *status);

/**
 *******************************************************************************
 * @brief       read protection status.
 * @param[in]   id instance id
 * @param[out]  status MPWM protection status
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_MPWM_ReadProtectionStatus(enum mpwm_id id, uint32_t *status);

/**
 *******************************************************************************
 * @brief       read prortion status.
 * @param[in]   id instance id
 * @param[out]  status MPWM over current status
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_MPWM_ReadOverCurrentStatus(enum mpwm_id id, uint32_t *status);

/**
 *******************************************************************************
 * @brief       enable MPWM output.
 * @param[in]   id instance id
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_MPWM_EnablePWMOutput(enum mpwm_id id);

/**
 *******************************************************************************
 * @brief       disable MPWM output.
 * @param[in]   id instance id
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_MPWM_DisablePWMOutput(enum mpwm_id id);


#endif /* _A34M456_MPWM_H_ */

/** @} */
/** @} */

