/**
 *******************************************************************************
 * @file        a34m456_pfcp.h
 * @author      ABOV R&D Division
 * @brief       power factor correction pwm driver
 *
 * Copyright 2024 ABOV Semiconductor Co.,Ltd. All rights reserved.
 *
 * This file is licensed under terms that are found in the LICENSE file
 * located at Document directory.
 * If this file is delivered or shared without applicable license terms,
 * the terms of the BSD-3-Clause license shall be applied.
 * Reference: https://opensource.org/licenses/BSD-3-Clause
 ******************************************************************************/

/**
 * @addtogroup  driver
 * @{
 * @defgroup    driver_pfcp PFCP
 * @{
 * @brief       PFCP driver
 */

#ifndef _A34M456_PFCP_H_
#define _A34M456_PFCP_H_

#ifdef __cplusplus
extern "C"
{
#endif

#include <stdint.h>
#include "a34m456_driver_common.h"

/**
 * PFCP instance id
 */
enum pfcp_id
{
    PFCP_ID_0   = 0,                        /**< PFCP */
    PFCP_ID_MAX = 1,                        /**< PFCP max */
};

/**
 * PFCP mode
 */
enum pfcp_mode
{
    PFCP_SINGLE_MODE              = 0,            /**< SINGLE PFC */
    PFCP_INTERLEAVED_MODE         = 1,            /**< INTERLEAVED_PFC */
    PFCP_COMPLEMENTARY_MODE       = 2,            /**< COMPLEMENTARY_PFC */
};

/**
 * PFCP count mode
 */
enum pfcp_count_mode
{
    UP_DOWN_COUNT_MODE           = 0,            /**< UPDOWN COUNT MODE */
    UP_COUNT_MODE                = 1,            /**< UP COUNT MODE */
};


/**
 * PFCP symmetric mode
 */
enum pfcp_asym_mode
{
    PFCP_SYMMETRIC_MODE    = 0,    /**< symmetric mode */
    PFCP_ASYMMETRIC_MODE   = 1,    /**< asymmetric mode */
};


/**
 * PFCP phase type
 */

enum pfcp_phase_type
{
    PFCP_PHASE_TYPE_A   = 0,                /**< PFCP phase A */
    PFCP_PHASE_TYPE_B   = 1,                /**< PFCP phase B */
};

/**
 * PFCP output type
 */
enum pfcp_output_type
{
    PFCP_OUTPUT_TYPE_A  = 0,                /**< PFCPAO*/
    PFCP_OUTPUT_TYPE_B  = 1,                /**< PFCPBO */
};

/**
 * PFCP duty type
 */
enum pfcp_duty_type
{
    PFCP_DUTY_TYPE_L  = 0,                /**< DAL, DBL*/
    PFCP_DUTY_TYPE_H  = 1,                /**< DAH, DBH */
};

/**
 * PFCP output level update setting 
 */
enum pfcp_output_lv_update_ao_ab
{
    PFCP_OUTPUT_LEVEL_SPECIFIED_TIME        = 0,        /**< Updates at specified timings */
    PFCP_OUTPUT_LEVEL_AT_ONCE               = 1,        /**< Updates all output levels at once */
};
	
/**
 * PFCPB output level update top(period) setting 
 */
enum pfcp_output_lv_update_top_b
{
    PFCP_OUTPUT_LEVEL_TOP_B_NOT_UPDATE        = 0,        /**< PFCP_OLR register is not updated at every period-match. */
    PFCP_OUTPUT_LEVEL_TOP_B_UPDATE            = 1,        /**< PFCP_OLR register is updated at every period-match. */
};
/**
 * PFCPB output level update bottom setting 
 */
enum pfcp_output_lv_update_btm_b
{
    PFCP_OUTPUT_LEVEL_BTM_B_NOT_UPDATE        = 0,        /**< PFCP_OLR register is not updated at every bottom-match. */
    PFCP_OUTPUT_LEVEL_BTM_B_UPDATE            = 1,        /**< PFCP_OLR register is updated at every bottom-match. */
};

/**
 * PFCPA output level update top(period) setting 
 */
enum pfcp_output_lv_update_top_a
{
    PFCP_OUTPUT_LEVEL_TOP_A_NOT_UPDATE        = 0,        /**< PFCP_OLR register is not updated at every period-match. */
    PFCP_OUTPUT_LEVEL_TOP_A_UPDATE            = 1,        /**< PFCP_OLR register is updated at every period-match. */
};
/**
 * PFCPA output level update bottom setting 
 */
enum pfcp_output_lv_update_btm_a
{
    PFCP_OUTPUT_LEVEL_BTM_A_NOT_UPDATE        = 0,        /**< PFCP_OLR register is not updated at every bottom-match. */
    PFCP_OUTPUT_LEVEL_BTM_A_UPDATE            = 1,        /**< PFCP_OLR register is updated at every bottom-match. */
};

	
/**
 * PFCP duty update setting 
 */
enum pfcp_duty_update_ao_ab
{
    PFCP_DUTY_SPECIFIED_TIME        = 0,        /**< Updates duty at specified timings */
    PFCP_DUTY_AT_ONCE               = 1,        /**< Updates duty at once */
};
	
/**
 * PFCPB duty update top(period) setting 
 */
enum pfcp_duty_update_top_b
{
    PFCP_DUTY_TOP_B_NOT_UPDATE        = 0,        /**< Duty value are not updated at every period-match. */
    PFCP_DUTY_TOP_B_UPDATE            = 1,        /**< Duty value are updated at every period-match. */
};
/**
 * PFCPB duty update bottom setting 
 */
enum pfcp_duty_update_btm_b
{
    PFCP_DUTY_BTM_B_NOT_UPDATE        = 0,        /**< Duty value are not updated at every bottom-match.. */
    PFCP_DUTY_BTM_B_UPDATE            = 1,        /**< Duty value are updated at every bottom-match. */
};

/**
 * PFCPA duty update top(period) setting 
 */
enum pfcp_duty_update_top_a
{
    PFCP_DUTY_TOP_A_NOT_UPDATE        = 0,        /**< Duty value are not updated at every period-match. */
    PFCP_DUTY_TOP_A_UPDATE            = 1,        /**< Duty value are updated at every period-match. */
};
/**
 * PFCPA duty update bottom setting 
 */
enum pfcp_duty_update_btm_a
{
    PFCP_DUTY_BTM_A_NOT_UPDATE        = 0,        /**< Duty value are not updated at every bottom-match.. */
    PFCP_DUTY_BTM_A_UPDATE            = 1,        /**< Duty value are updated at every bottom-match. */
};

	
/**
 * PFCP period update top(period) setting 
 */
enum pfcp_period_update_top
{
    PFCP_PRD_TOP_NOT_UPDATE        = 0,        /**< Period value are not updated at every period-match. */
    PFCP_PRD_TOP_UPDATE            = 1,        /**< Period value are updated at every period-match. */
};
/**
 * PFCP period update bottom setting 
 */
enum pfcp_period_update_btm
{
    PFCP_PRD_BTM_NOT_UPDATE        = 0,        /**< Period value are not updated at every bottom-match.. */
    PFCP_PRD_BTM_UPDATE            = 1,        /**< Period value are updated at every bottom-match. */
};
/**
 * PFCP dead time mode
 */
enum pfcp_dead_time_mode
{
    PFCP_DEAD_TIME_MODE_L_EDGE   = 0,        /**< Inserts dead time at the leading edge of PFCPAO and the trailing edge of PFCPBO */
    PFCP_DEAD_TIME_MODE_T_EDGE   = 1,        /**< Inserts dead time at the trailing edge of PFCPAO and the leading edge of PFCPBO */
};

/**
 * PFCP trigger souce of ADC
 */

enum pfcp_adc_trigger_source
{
    PFCP_ADC_TRIGGER_SOURCE_A_COUNTER   = 0,    /**< Set PFCP phase A to trigger of ADC */
    PFCP_ADC_TRIGGER_SOURCE_B_COUNTER   = 1,    /**< Set PFCP phase B to trigger of ADC */
};
	
/**
 * PFCP trigger register update mode
 */
enum pfcp_adc_trigger_update
{
    PFCP_ADC_TRIGGER_UPDATE_PERIOD      = 0,    /**< ADC trigger value applied at period match event */
    PFCP_ADC_TRIGGER_UPDATE_2CLOCK      = 1,    /**< written trigger register values are sent to the trigger compare block after two PWM clock cycles */
};

/**
 * PFCP trigger mode of ADC
 */
enum pfcp_adc_trigger_mode
{
    PFCP_ADC_TRIGGER_MODE_DISABLE       = 0,    /**< Disable trigger mode */
    PFCP_ADC_TRIGGER_MODE_UP_COUNT      = 1,    /**< Set up count match trigger mode */
    PFCP_ADC_TRIGGER_MODE_DOWN_COUNT    = 2,    /**< Set down count match trigger mode */
    PFCP_ADC_TRIGGER_MODE_UPDOWN_COUNT  = 3,    /**< Set up-down count match trigger mode */
};

/**
 * PFCP trigger id of ADC
 */
enum pfcp_trgger_id
{
    PFCP_TRGGER_ID_1    = 0,            /**< PFCP Trigger 1 */
    PFCP_TRGGER_ID_2    = 1,            /**< PFCP Trigger 2 */
    PFCP_TRGGER_ID_3    = 2,            /**< PFCP Trigger 3 */
    PFCP_TRGGER_ID_4    = 3,            /**< PFCP Trigger 4 */
    PFCP_TRGGER_ID_5    = 4,            /**< PFCP Trigger 5 */
    PFCP_TRGGER_ID_6    = 5,            /**< PFCP Trigger 6 */
    PFCP_TRGGER_ID_MAX  = 6,            /**< PFCP Trigger max */
};

/**
 * PFCP interrupt event type
 */
enum pfcp_irq_event_type
{
    PFCP_IRQ_EVENT_TYPE_PRT0     = 1,    /**< IRQ event protection0 */
    PFCP_IRQ_EVENT_TYPE_PRT1      = 2,    /**< IRQ event protection1 */
    PFCP_IRQ_EVENT_TYPE_PFCP    = 4,    /**< IRQ event PFCP(bottom/period/duty/ATR) */
};


/**
 * PFCPB interrupt period
 */
enum pfcp_prdie_b
{
    PFCP_PRDIE_B_DISABLE     = 0,
    PFCP_PRDIE_B_ENABLE      = 1,
};
/**
 * PFCPB interrupt bottom
 */
enum pfcp_botie_b
{
    PFCP_BOTIE_B_DISABLE     = 0,
    PFCP_BOTIE_B_ENABLE      = 1,
};

/**
 * PFCPA interrupt period
 */
enum pfcp_prdie_a
{
    PFCP_PRDIE_A_DISABLE     = 0,
    PFCP_PRDIE_A_ENABLE      = 1,
};
/**
 * PFCPA interrupt bottom
 */
enum pfcp_botie_a
{
    PFCP_BOTIE_A_DISABLE     = 0,
    PFCP_BOTIE_A_ENABLE      = 1,
};

	
/**
 * PFCP interrupt atr6ie
 */
enum pfcp_atr6ie
{
    PFCP_ATR6IE_DISABLE     = 0,
    PFCP_ATR6IE_ENABLE      = 1,
};
/**
 * PFCP interrupt atr5ie
 */
enum pfcp_atr5ie
{
    PFCP_ATR5IE_DISABLE     = 0,
    PFCP_ATR5IE_ENABLE      = 1,
};
/**
 * PFCP interrupt atr4ie
 */
enum pfcp_bhie_atr4ie
{
    PFCP_ATR4IE_DISABLE     = 0,
    PFCP_ATR4IE_ENABLE      = 1,
};
/**
 * PFCP interrupt atr3ie
 */
enum pfcp_blie_atr3ie
{
    PFCP_ATR3IE_DISABLE     = 0,
    PFCP_ATR3IE_ENABLE      = 1,
};
/**
 * PFCP interrupt atr2ie
 */
enum pfcp_ahie_atr2ie
{
    PFCP_ATR2IE_DISABLE     = 0,
    PFCP_ATR2IE_ENABLE      = 1,
};
/**
 * PFCP interrupt atr1ie
 */
enum pfcp_alie_atr1ie
{
    PFCP_ATR1IE_DISABLE     = 0,
    PFCP_ATR1IE_ENABLE      = 1,
};
/**
 * PFCP Dead time function enable
 */
enum pfcp_dten
{
    PFCP_DT_DISABLE     = 0,
    PFCP_DT_ENABLE      = 1,
};
/**
 * PFCP PSHRT enable
 */
enum pfcp_pshrt
{
    PFCP_PSHRT_DISABLE     = 0,
    PFCP_PSHRT_ENABLE      = 1,
};
/**
 * PFCP HDTOL. output level at high side dead time 
 */
enum pfcp_adtol
{
    PFCP_ADTOL_LOW_LEVEL     = 0,
    PFCP_ADTOL_HIGH_LEVEL    = 1,
};
/**
 * PFCP LDTOL. output level at low side dead time 
 */
enum pfcp_bdtol
{
    PFCP_BDTOL_LOW_LEVEL     = 0,
    PFCP_BDTOL_HIGH_LEVEL    = 1,
};
/**
 * PFCP dead clock div
 */
enum pfcp_dead_time_clk
{
    PFCP_DEAD_TIME_CLK_PWM_DIV_2    = 0,    /**< PFCP clock devide 2 */
    PFCP_DEAD_TIME_CLK_PWM_DIV_4    = 1,    /**< PFCP clock devide 4 */
    PFCP_DEAD_TIME_CLK_PWM_DIV_8    = 2,    /**< PFCP clock devide 8 */
    PFCP_DEAD_TIME_CLK_PWM_DIV_16   = 3,    /**< PFCP clock devide 16 */
};

/**
 * PFCP protection0 & protection1 config polarity
 */
enum pfcp_prt_polarity
{
    PFCP_PRT_POLARITY_LOW        = 0,    /**< L active */
    PFCP_PRT_POLARITY_HIGH       = 1,    /**< H active */
};
/**
 * PFCP protection0 debounce clock divide
 */
enum pfcp_prt0
{
    PFCP_PRT0_DISABLE       = 0,    /**< Disable */
    PFCP_PRT0_ENABLE        = 1,    /**< Enable */
};
/**
 * PFCP protection1 debounce clock divide
 */
enum pfcp_prt1
{
    PFCP_PRT1_DISABLE       = 0,    /**< Disable */
    PFCP_PRT1_ENABLE        = 1,    /**< Enable */
};
/**
 *******************************************************************************
 * @brief       PFCP interrupt callback function type
 * @param[in]   event event type (::pfcp_irq_event_type)
 * @param[in]   context context provided during initialization
 ******************************************************************************/
typedef void (*pfcp_irq_handler_t)(uint32_t evnet, void *context);

/**
 * PFCP interrupt configuration
 */
struct pfcp_irq_cfg
{
    enum pfcp_prdie_b          prdie_b;             /**< PFCPB Period interrupt Enable */
    enum pfcp_botie_b          botie_b;             /**< PFCPB Bottom interrupt Enable */
    enum pfcp_prdie_a          prdie_a;             /**< PFCPA Period interrupt Enable */
    enum pfcp_botie_a          botie_a;             /**< PFCPA Bottom interrupt Enable */
    enum pfcp_atr6ie           atr6ie;              /**< PFCP ATR6 interrupt Enable */
    enum pfcp_atr5ie           atr5ie;              /**< PFCP ATR5 interrupt Enable */
    enum pfcp_bhie_atr4ie      atr4ie;              /**< PFCP BH duty match or ATR4 interrupt Enable */
    enum pfcp_blie_atr3ie      atr3ie;              /**< PFCP BL duty match or ATR3 interrupt Enable */
    enum pfcp_ahie_atr2ie      atr2ie;              /**< PFCP AH duty match or ATR2 interrupt Enable */
    enum pfcp_alie_atr1ie      atr1ie;              /**< PFCP AL duty match or ATR1 interrupt Enable */
    uint8_t                  irq_prio;              /**< interrupt priority */
};

/**
 * PFCP Mode register configuration
 */
struct pfcp_cfg
{
    enum pfcp_mode                          mode;                /**< PFCP mode */
    enum pfcp_count_mode                    count_mode;          /**< PFCP count mode */
    enum pfcp_asym_mode                     asym_mode;           /**< PFCP asym mode */
    enum pfcp_output_lv_update_ao_ab        out_lv_upd_ao_ab;    /**< PFCPAB output level update at once */
    enum pfcp_output_lv_update_top_b        out_lv_upd_top_b;    /**< PFCPB top update output level */
    enum pfcp_output_lv_update_btm_b        out_lv_upd_btm_b;    /**< PFCPB bottom update output level */
    enum pfcp_output_lv_update_top_a        out_lv_upd_top_a;    /**< PFCPA top update output level */
    enum pfcp_output_lv_update_btm_a        out_lv_upd_btm_a;    /**< PFCPA bottom update output level */
    enum pfcp_duty_update_ao_ab             d_upd_ao_ab;         /**< PFCPAB duty update at once */
    enum pfcp_duty_update_top_b             d_upd_top_b;         /**< PFCPB duty top update */
    enum pfcp_duty_update_btm_b             d_upd_btm_b;         /**< PFCPB duty bottom update */
    enum pfcp_duty_update_top_a             d_upd_top_a;         /**< PFCPA duty top update */
    enum pfcp_duty_update_btm_a             d_upd_btm_a;         /**< PFCPA duty bottom update */
    enum pfcp_period_update_top             p_upd_top;           /**< PFCP period top update */
    enum pfcp_period_update_btm             p_upd_btm;           /**< PFCP period bottom update */
    uint32_t clk_div;                                            /**< 4bit clock divider */
};

/**
 * PFCP input protection0 configuration
 */
struct pfcp_input_prt0_cfg
{
    enum pfcp_prt_polarity     polarity;              /**< input polarity (0:L-active, 1:H-active) */
    enum pfcp_prt0        enable_prt7;       /**< enable or disable ADC2 comparator 0 protection  */
    enum pfcp_prt0        enable_prt6;       /**< enable or disable ADC1 comparator 0 protection  */
    enum pfcp_prt0        enable_prt5;       /**< enable or disable ADC0 comparator 0 protection  */
    enum pfcp_prt0        enable_prt4;       /**< enable or disable CMP3 protection  */
    enum pfcp_prt0        enable_prt3;       /**< enable or disable CMP2 protection  */
    enum pfcp_prt0        enable_prt2;       /**< enable or disable CMP1 protection  */
    enum pfcp_prt0        enable_prt1;       /**< enable or disable CMP0 protection  */
    enum pfcp_prt0        enable_prt0;       /**< enable or disable PRTIN0 protection  */
    uint32_t clk_div;           /**< 3bit debounce clock divider */
    uint32_t debounce;          /**< debounce time (0:no debounce, 0x01 ~ 0xFF:PFCP clock * debounce, ) */
};

/**
 * PFCP input protection1 configuration
 */
struct pfcp_input_prt1_cfg
{
    enum pfcp_prt_polarity     polarity;              /**< input polarity (0:L-active, 1:H-active) */
    enum pfcp_prt1        enable_prt7;       /**< enable or disable ADC2 comparator 0 protection  */
    enum pfcp_prt1        enable_prt6;       /**< enable or disable ADC1 comparator 0 protection  */
    enum pfcp_prt1        enable_prt5;       /**< enable or disable ADC0 comparator 0 protection  */
    enum pfcp_prt1        enable_prt4;       /**< enable or disable CMP3 protection  */
    enum pfcp_prt1        enable_prt3;       /**< enable or disable CMP2 protection  */
    enum pfcp_prt1        enable_prt2;       /**< enable or disable CMP1 protection  */
    enum pfcp_prt1        enable_prt1;       /**< enable or disable CMP0 protection  */
    enum pfcp_prt1        enable_prt0;       /**< enable or disable PRTIN1 protection  */
    uint32_t clk_div;           /**< 3bit debounce clock divider */
    uint32_t debounce;          /**< debounce time (0:no debounce, 0x01 ~ 0xFF:PFCP clock * debounce, ) */
};

/**
 * PFCP deatime configuration
 */
struct pfcp_dt_cfg
{
    enum pfcp_dten              enable;                     /**< enable or disable dead time function */
    enum pfcp_pshrt             short_circuit_protect_en;   /**< enable of disable short circuit protection */
    enum pfcp_dead_time_clk     clk;                        /**< prescalter of dead time */
    enum pfcp_bdtol             output_level_bside;      /**< dead time output level of high side */
    enum pfcp_adtol             output_level_aside;       /**< dead time output level of low side */
    uint32_t                    bside_dead_time;            /**< high side dead time value */
    uint32_t                    aside_dead_time;            /**< low side dead time value */
};

/**
 * PFCP ADC trigger configuration
 */
struct pfcp_adc_tr_cfg
{
    uint32_t                        reg_mode;       /**< ADC register update mode */
    enum pfcp_adc_trigger_mode      mode;           /**< ADC trigger mode */
    uint32_t                        tr_count;       /**< ADC trigger count value */
};

/*
#define PFCP_SR_ALIF_ATR1IF            (0x01UL << PFCP_SR_ALIF_ATR1IF_Pos)

#define PFCP_FOLR_FOLKEY            (0xCA)
*/

typedef enum
{
    PFCP_PROTECTION_PROT0,
    PFCP_PROTECTION_PROT1,
} PFCP_PROTECTION_Type;

/**
 *******************************************************************************
 * @brief       Initialize PFCP
 * @param[in]   id instance id
 * @param[in]   cfg configuration
 * @param[in]   handler callback function to receive event
 * @param[in]   context caller's context
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_PFCP_Init(enum pfcp_id id, struct pfcp_cfg *cfg,
                      pfcp_irq_handler_t handler,
                      void *context);

/**
 *******************************************************************************
 * @brief       Deintialize PFCP.
 * @param[in]   id instance id
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_PFCP_Deinit(enum pfcp_id id);

/**
 *******************************************************************************
 * @brief       configure PFCP interrupt type.
 * @param[in]   id instance id
 * @param[in]   cfg cofigureation
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_PFCP_SetIRQConfig(enum pfcp_id id, struct pfcp_irq_cfg *cfg);

/**
 *******************************************************************************
 * @brief       configure PFCP input protection0.
 * @param[in]   id instance id
 * @param[in]   handler call back lable
 * @param[in]   cfg cofigureation
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_PFCP_SetProtection0InputConfig(enum pfcp_id id, pfcp_irq_handler_t handler,
                                         struct pfcp_input_prt0_cfg *cfg);

/**
 *******************************************************************************
 * @brief       configure PFCP output protection.
 * @param[in]   id instance id
 * @param[in]   handler call back lable
 * @param[in]   ot PFCP output type
 * @param[in]   en enable
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_PFCP_SetProtection0OutputConfig(enum pfcp_id id, pfcp_irq_handler_t handler,
                                           enum pfcp_output_type ot, uint32_t en);

/**
 *******************************************************************************
 * @brief       configure PFCP input protection1.
 * @param[in]   id instance id
 * @param[in]   handler call back lable
 * @param[in]   cfg cofigureation
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_PFCP_SetProtection1InputConfig(enum pfcp_id id, pfcp_irq_handler_t handler,
                                           struct pfcp_input_prt1_cfg *cfg);

/**
 *******************************************************************************
 * @brief       configure PFCP output protection.
 * @param[in]   id instance id
 * @param[in]   handler call back lable
 * @param[in]   ot PFCP output type
 * @param[in]   en enable
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_PFCP_SetProtection1OutputConfig(enum pfcp_id id, pfcp_irq_handler_t handler,
                                            enum pfcp_output_type ot, uint32_t en);

/**
 *******************************************************************************
 * @brief       configure PFCP dead time.
 * @param[in]   id instance id
 * @param[in]   p_type PFCP phase type
 * @param[in]   cfg cofigureation
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_PFCP_SetDeadTimeConfig(enum pfcp_id id, enum pfcp_phase_type p_type, struct pfcp_dt_cfg *cfg);

/**
 *******************************************************************************
 * @brief       configure PFCP trigger of ADC.
 * @param[in]   id instance id
 * @param[in]   t_id PFCP trigger id
 * @param[in]   cfg cofigureation
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_PFCP_SetADCTriggerConfig(enum pfcp_id id, enum pfcp_trgger_id t_id, struct pfcp_adc_tr_cfg *cfg);

/**
 *******************************************************************************
 * @brief       set PFCP IRQ interval..
 * @param[in]   id instance id
 * @param[in]   p_type PFCP phase type
 * @param[in]   irq_intv IRQ interval value
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_PFCP_SetIRQinterval(enum pfcp_id id, enum pfcp_phase_type p_type,
                                uint32_t irq_intv);

/**
 *******************************************************************************
 * @brief       control PFCP output.
 * @param[in]   id instance id
 * @param[in]   p_type PFCP phase type
 * @param[in]   ot PFCP output type
 * @param[in]   en enable(1) or disable(0)
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_PFCP_ControlPWMOutput(enum pfcp_id id,
                                  enum pfcp_output_type ot, uint32_t en);

/**
 *******************************************************************************
 * @brief       set PFCP output level at disable phase.
 * @param[in]   id instance id
 * @param[in]   p_type PFCP phase type
 * @param[in]   ot PFCP output type
 * @param[in]   hl high(1) or low(0)
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_PFCP_SetDisabledOutputLevel(enum pfcp_id id,
                                        enum pfcp_output_type ot, uint32_t hl);

/**
 *******************************************************************************
 * @brief       set PFCP start output level.
 * @param[in]   id instance id
 * @param[in]   p_type PFCP phase type
 * @param[in]   ot PFCP output type
 * @param[in]   hl high(1) or low(0)
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_PFCP_SetStartOutputLevel(enum pfcp_id id,
                                        enum pfcp_output_type ot, uint32_t hl);

/**
 *******************************************************************************
 * @brief       set PFCP force output level.
 * @param[in]   id instance id
 * @param[in]   p_type PFCP phase type
 * @param[in]   ot PFCP output type
 * @param[in]   hl high(1) or low(0)
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_PFCP_SetForceOutputLevel(enum pfcp_id id,
                                          enum pfcp_output_type ot, uint8_t hl);

/**
 *******************************************************************************
 * @brief       set PFCP period.
 * @param[in]   id instance id
 * @param[in]   p_type PFCP phase type
 * @param[in]   prd period value
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_PFCP_SetPeriod(enum pfcp_id id, uint16_t prd);

/**
 *******************************************************************************
 * @brief       set PFCP duty.
 * @param[in]   id instance id
 * @param[in]   p_type PFCP phase type
 * @param[in]   ot PFCP output type
 * @param[in]   duty period value
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_PFCP_SetDuty(enum pfcp_id id, enum pfcp_output_type ot,
                         enum pfcp_duty_type d_type, uint16_t duty);

/**
 *******************************************************************************
 * @brief       start PFCP.
 * @param[in]   id instance id
 * @param[in]   p_type PFCP phase type
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_PFCP_Start(enum pfcp_id id, enum pfcp_phase_type p_type);

/**
 *******************************************************************************
 * @brief       stop PFCP.
 * @param[in]   id instance id
 * @param[in]   p_type PFCP phase type
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_PFCP_Stop(enum pfcp_id id, enum pfcp_phase_type p_type);

/**
 *******************************************************************************
 * @brief       Halt PFCP.
 * @param[in]   id instance id
 * @param[in]   p_type PFCP phase type
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_PFCP_Halt(enum pfcp_id id, enum pfcp_phase_type p_type);

/**
 *******************************************************************************
 * @brief       Resume PFCP.
 * @param[in]   id instance id
 * @param[in]   p_type PFCP phase type
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_PFCP_Resume(enum pfcp_id id, enum pfcp_phase_type p_type);

/**
 *******************************************************************************
 * @brief       read count.
 * @param[in]   id instance id
 * @param[in]   p_type PFCP phase type
 * @param[out]  count PFCP count
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_PFCP_ReadCount(enum pfcp_id id, enum pfcp_output_type ot, uint16_t *count);

/**
 *******************************************************************************
 * @brief       read status.
 * @param[in]   id instance id
 * @param[out]  status PFCP status
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_PFCP_ReadStatus(enum pfcp_id id, uint32_t *status);

/**
 *******************************************************************************
 * @brief       read protection status.
 * @param[in]   id instance id
 * @param[out]  status PFCP protection status
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_PFCP_ReadProtectionStatus(enum pfcp_id id, uint32_t *status);

/**
 *******************************************************************************
 * @brief       read prortion status.
 * @param[in]   id instance id
 * @param[out]  status PFCP over current status
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_PFCP_ReadOverCurrentStatus(enum pfcp_id id, uint32_t *status);

/**
 *******************************************************************************
 * @brief       enable PFCP output.
 * @param[in]   id instance id
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_PFCP_EnablePWMOutput(enum pfcp_id id);

/**
 *******************************************************************************
 * @brief       disable PFCP output.
 * @param[in]   id instance id
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_PFCP_DisablePWMOutput(enum pfcp_id id);


#endif /* _A34M456_PFCP_H_ */

/** @} */
/** @} */

