/**
 *******************************************************************************
 * @file        a34m456_pwr.h
 * @author      ABOV R&D Division
 * @brief       Power Header File
 *
 * Copyright 2024 ABOV Semiconductor Co.,Ltd. All rights reserved.
 *
 * This file is licensed under terms that are found in the LICENSE file
 * located at Document directory.
 * If this file is delivered or shared without applicable license terms,
 * the terms of the BSD-3-Clause license shall be applied.
 * Reference: https://opensource.org/licenses/BSD-3-Clause
 ******************************************************************************/

/**
 * @addtogroup  driver
 * @{
 * @defgroup    driver_pwr PWR
 * @{
 * @brief       PWR driver
 */


#ifndef _A34M456_PWR_H_
#define _A34M456_PWR_H_

#ifdef __cplusplus
extern "C"
{
#endif

#include <stdint.h>
#include "a34m456_driver_common.h"

/**
 * low voltage indicator level selection
 */
enum pwr_lvi_level
{
    PWR_LVI_2_20V    = (0x6uL << 0),
    PWR_LVI_2_32V    = (0x7uL << 0),
    PWR_LVI_2_46V    = (0x8uL << 0),
    PWR_LVI_2_61V    = (0x9uL << 0),
    PWR_LVI_2_79V    = (0xAuL << 0),
    PWR_LVI_2_99V    = (0xBuL << 0),
    PWR_LVI_3_22V    = (0xCuL << 0),
    PWR_LVI_3_48V    = (0xDuL << 0),
    PWR_LVI_3_80V    = (0xEuL << 0),
    PWR_LVI_4_18V    = (0xFuL << 0),
};

/**
 * low voltage reset level selection
 */
enum pwr_lvr_level
{
    PWR_LVR_2_20V    = (0x6uL << 0),
    PWR_LVR_2_32V    = (0x7uL << 0),
    PWR_LVR_2_46V    = (0x8uL << 0),
    PWR_LVR_2_61V    = (0x9uL << 0),
    PWR_LVR_2_79V    = (0xAuL << 0),
    PWR_LVR_2_99V    = (0xBuL << 0),
    PWR_LVR_3_22V    = (0xCuL << 0),
    PWR_LVR_3_48V    = (0xDuL << 0),
    PWR_LVR_3_80V    = (0xEuL << 0),
    PWR_LVR_4_18V    = (0xFuL << 0),

};

/**
 * low voltage event type
 */
enum pwr_event
{
    PWR_EVENT_LVI,                      /**< low voltage interrupt */
};

/**
 * low voltage interrupt configuration
 */
struct lvi_cfg
{
    uint8_t             auto_off;       /**< automatically turn off when entering deep sleep */
    enum pwr_lvi_level   level;          /**< LVI level */
    uint8_t             irq_prio;       /**< interrupt priority */
};

/**
 * low voltage reset configuration
 */
struct lvr_cfg
{
    uint8_t             auto_off;       /**< automatically turn off when entering deep sleep */
    enum pwr_lvr_level   level;          /**< LVR level */
    uint8_t             irq_prio;       /**< interrupt priority */
};
/**
 *******************************************************************************
 * @brief       PWR interrupt callback function type
 * @param[in]   event event type ::pwr_event
 * @param[in]   context context provided during initialization
 ******************************************************************************/
typedef void (*pwr_irq_handler_t)(uint32_t event, void *context);

/**
 *******************************************************************************
 * @brief       Initialize LVI.
 *              By initializing, LVI gets enabled.
 * @param[in]   cfg configuration
 * @param[in]   handler callback function to receive event
 * @param[in]   context caller's context
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_LVI_Init(struct lvi_cfg *cfg,
                     pwr_irq_handler_t handler,
                     void *context);

/**
 *******************************************************************************
 * @brief       Deinitialize LVI
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_LVI_Deinit(void);

/**
 *******************************************************************************
 * @brief       Initialize LVR.
 *              By initializing, LVR gets enabled.
 * @param[in]   cfg configuration
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_LVR_Init(struct lvr_cfg *cfg);

/**
 *******************************************************************************
 * @brief       Deinitialize LVR
 * @return      driver error code
 ******************************************************************************/
int32_t HAL_LVR_Deinit(void);

/**
 *******************************************************************************
 * @brief       SleepMode
 * @return      none
 ******************************************************************************/
void HAL_PWR_EnterSleepMode(void);

/**
 *******************************************************************************
 * @brief       DeepSleep Mode
 * @return      none
 ******************************************************************************/
void HAL_PWR_EnterDeepSleepMode(void);



#ifdef __cplusplus
}
#endif

#endif /* _A34m456_PWR_H_ */

/** @} */
/** @} */

