/**
 *******************************************************************************
 * @file        a34m456_cmp.c
 * @author      ABOV R&D Division
 * @brief       cmp driver
 *
 * Copyright 2024 ABOV Semiconductor Co.,Ltd. All rights reserved.
 *
 * This file is licensed under terms that are found in the LICENSE file
 * located at Document directory.
 * If this file is delivered or shared without applicable license terms,
 * the terms of the BSD-3-Clause license shall be applied.
 * Reference: https://opensource.org/licenses/BSD-3-Clause
 ******************************************************************************/

#include <stdint.h>
#include <string.h>

#include "a34m456.h"
#include "a34m456_scu.h"
#include "a34m456_cmp.h"

#define CMP_REG_OFFSET     0x10
#define CMP_CHECK_START_UP(STATUS)      while((STATUS & CMP_STAT_CMPBUSY_Msk) == 0x200UL)

static __INLINE CMP_Type *CMP_REG(enum cmp_id id)
{
    return (CMP_Type *)(CMP0_BASE + (CMP_REG_OFFSET * id));
}

static __INLINE IRQn_Type CMP_IRQ_NUM(enum cmp_id id)
{
    return (IRQn_Type)((uint8_t)CMP0_IRQn + id);
}

struct cmp_ctrl_block
{
    cmp_irq_handler_t  handler;
    void *              context;
    uint8_t             irq_enable;
};

static uint32_t g_cmp_ref_count = 0;
static struct cmp_ctrl_block g_ccb[CMP_ID_MAX];

int32_t HAL_CMP_Init(enum cmp_id id, struct cmp_cfg *cfg,
                      cmp_irq_handler_t handler,
                      void *context)
{
    struct cmp_ctrl_block *ccb;
    CMP_Type *cmp;

    DRIVER_ASSERT(id < CMP_ID_MAX);
    DRIVER_ASSERT(cfg != NULL);

    if (g_cmp_ref_count == CMP_ID_MAX)
    {
        return DRIVER_ERROR_NO_RESOURCE;
    }

    ccb = &g_ccb[id];
    cmp = CMP_REG(id);

    memset(ccb, 0, sizeof(struct cmp_ctrl_block));

    ccb->handler = handler;
    ccb->context = context;

    if (g_cmp_ref_count == 0)
    {
        SCU->PER2 &= ~(0x01UL << SCU_PER2_CMP_Pos);
        SCU->PCER2 &= ~(0x01UL << SCU_PER2_CMP_Pos);

        SCU->PER2 |= (0x01UL << SCU_PER2_CMP_Pos);
        SCU->PCER2 |= (0x01UL << SCU_PER2_CMP_Pos);
    }

    g_cmp_ref_count++;
    cmp->CONF = 0;

    cmp->CONF |= (cfg->filter_div << CMP_CONF_FLTDIV_Pos);

    cmp->CONF |= (cfg->filter_count << CMP_CONF_FLTSEL_Pos);

    if (cfg->hystresis == CMP_HYSTERESIS_40MV)
    {
        cmp->CONF |= (1 << CMP_CONF_HYSEN_Pos) | (0 << CMP_CONF_HYSSEL_Pos);
    }
    else if (cfg->hystresis == CMP_HYSTERESIS_80MV)
    {
        cmp->CONF |= (1 << CMP_CONF_HYSEN_Pos) | (1 << CMP_CONF_HYSSEL_Pos);
    }
    else if (cfg->hystresis == CMP_HYSTERESIS_120MV)
    {
        cmp->CONF |= (1 << CMP_CONF_HYSEN_Pos) | (2 << CMP_CONF_HYSSEL_Pos);
    }
    else if (cfg->hystresis == CMP_HYSTERESIS_160MV)
    {
        cmp->CONF |= (1 << CMP_CONF_HYSEN_Pos) | (3 << CMP_CONF_HYSSEL_Pos);
    }
    else
    {
        cmp->CONF |= (0 << CMP_CONF_HYSEN_Pos) | (0 << CMP_CONF_HYSSEL_Pos);
    }

    if (cfg->irq_type == CMP_IRQ_LEVEL)
    {
        cmp->CONF |= (1 << CMP_CONF_INTTYPE_Pos);
        ccb->irq_enable = 1;
    }
    else if (cfg->irq_type == CMP_IRQ_LOW_EDGE)
    {
        cmp->CONF |= (2 << CMP_CONF_INTTYPE_Pos);
        ccb->irq_enable = 1;
    }
    else if (cfg->irq_type == CMP_IRQ_HIGH_EDGE)
    {
        cmp->CONF |= (2 << CMP_CONF_INTTYPE_Pos) | (1 << CMP_CONF_INTPOL_Pos);
        ccb->irq_enable = 1;
    }
    else if (cfg->irq_type == CMP_IRQ_BOTH_EDGE)
    {
        cmp->CONF |= (3 << CMP_CONF_INTTYPE_Pos);
        ccb->irq_enable = 1;
    }
    else if (cfg->irq_type == CMP_IRQ_NONE)
    {
        ccb->irq_enable = 0;
    }

    cmp->CONF |= (cfg->input_neg << CMP_CONF_CINNSEL_Pos);
    cmp->CONF |= (cfg->input_pos << CMP_CONF_CINPSEL_Pos);

    NVIC_ClearPendingIRQ(CMP_IRQ_NUM(id));
    NVIC_SetPriority(CMP_IRQ_NUM(id), cfg->irq_prio);
    NVIC_EnableIRQ(CMP_IRQ_NUM(id));

    return DRIVER_ERROR_OK;
}

int32_t HAL_CMP_Deinit(enum cmp_id id)
{
    DRIVER_ASSERT(id < CMP_ID_MAX);

    if (g_cmp_ref_count == 0)
    {
        return DRIVER_ERROR_NO_RESOURCE;
    }

    g_cmp_ref_count--;

    if (g_cmp_ref_count == 0)
    {
        SCU->PER2 &= ~(0x01UL << SCU_PER2_CMP_Pos);
        SCU->PCER2 &= ~(0x01UL << SCU_PCER2_CMP_Pos);
    }

    NVIC_DisableIRQ(CMP_IRQ_NUM(id));

    return DRIVER_ERROR_OK;
}

int32_t HAL_CMP_Start(enum cmp_id id)
{
    struct cmp_ctrl_block *ccb;
    CMP_Type *cmp;

    DRIVER_ASSERT(id < CMP_ID_MAX);

    cmp = CMP_REG(id);
    ccb = &g_ccb[id];

    cmp->CTRL |= (1 << CMP_CTRL_CMPEN_Pos);

    CMP_CHECK_START_UP(cmp->STAT);

    if (ccb->irq_enable)
    {
        cmp->CTRL |= (1 << CMP_CTRL_CMPINTEN_Pos);
    }

    return DRIVER_ERROR_OK;
}

int32_t HAL_CMP_Stop(enum cmp_id id)
{
    CMP_Type *cmp;

    DRIVER_ASSERT(id < CMP_ID_MAX);

    cmp = CMP_REG(id);

    cmp->CTRL &= ~(1 << CMP_CTRL_CMPEN_Pos);
    cmp->CTRL &= ~(1 << CMP_CTRL_CMPINTEN_Pos);

    return DRIVER_ERROR_OK;
}

int32_t HAL_CMP_GetStatus(enum cmp_id id, uint8_t *status)
{
    CMP_Type *cmp;
    uint32_t value;

    DRIVER_ASSERT(id < CMP_ID_MAX);

    cmp = CMP_REG(id);

    value = cmp->STAT;
    cmp->STAT = value;

    if (value & CMP_STAT_CMPFLAG_Msk)
    {
        *status = CMP_STATUS_UP;
    }
    else
    {
        *status = CMP_STATUS_DOWN;
    }

    return DRIVER_ERROR_OK;
}


void CMP_IRQHandler(enum cmp_id id)
{
    struct cmp_ctrl_block *ccb;
    CMP_Type *cmp;
    uint32_t status;
    uint32_t event;

    ccb = &g_ccb[id];
    cmp = CMP_REG(id);

    status = cmp->STAT;
    cmp->STAT = status;
    if (status & CMP_STAT_CMPFLAG_Msk)
    {
        event = CMP_EVENT_UP;
    }
    else
    {
        event = CMP_EVENT_DOWN;
    }

    ccb->handler(event, ccb->context);
}

void CMP0_IRQHandler(void)
{
    CMP_IRQHandler(CMP_ID_0);
}

void CMP1_IRQHandler(void)
{
    CMP_IRQHandler(CMP_ID_1);
}

void CMP2_IRQHandler(void)
{
    CMP_IRQHandler(CMP_ID_2);
}

void CMP3_IRQHandler(void)
{
    CMP_IRQHandler(CMP_ID_3);
}
