/**
 *******************************************************************************
 * @file        a34m456_dac.c
 * @author      ABOV R&D Division
 * @brief       ADC driver
 *
 * Copyright 2024 ABOV Semiconductor Co.,Ltd. All rights reserved.
 *
 * This file is licensed under terms that are found in the LICENSE file
 * located at Document directory.
 * If this file is delivered or shared without applicable license terms,
 * the terms of the BSD-3-Clause license shall be applied.
 * Reference: https://opensource.org/licenses/BSD-3-Clause
 ******************************************************************************/

#include <string.h>
#include "a34m456.h"
#include "a34m456_dac.h"
#include "a34m456_scu.h"

#define DAC_REG_OFFSET  0x100

static __INLINE DAC_Type *DAC_REG(enum dac_id id)
{
    return (DAC_Type *)(DAC0_BASE + (DAC_REG_OFFSET * id));
}

struct dac_ctrl_block
{
    enum dac_out       out;
};

static struct dac_ctrl_block g_dcb[DAC_ID_MAX];

int32_t HAL_DAC_Init(enum dac_id id, struct dac_cfg *cfg)
{
    struct dac_ctrl_block *dcb;
    DAC_Type *dac;
    uint32_t reg_val=0;

    DRIVER_ASSERT(id < DAC_ID_MAX);
    DRIVER_ASSERT(cfg != NULL);

    dcb = &g_dcb[id];

    SCU->PER2 &= ~(1 << (SCU_PER2_DAC0_Pos + id));
    SCU->PCER2 &= ~(1 << (SCU_PCER2_DAC0_Pos + id));

    SCU->PER2 |= (1 << (SCU_PER2_DAC0_Pos + id));
    SCU->PCER2 |= (1 << (SCU_PCER2_DAC0_Pos + id));

    dac = DAC_REG(id);

    dcb->out = cfg->out;

    if (dcb->out == DAC_OUT_ENABLE){
         reg_val |= (0x01UL << DAC_CR_PORTOUT_EN_Pos);
    }
    else 
    {
        reg_val &= ~(0x01UL << DAC_CR_PORTOUT_EN_Pos);
    }

    dac->CR = reg_val;

    return DRIVER_ERROR_OK;
}

int32_t HAL_DAC_Start(enum dac_id id)
{
    uint32_t Count;
    volatile uint32_t i;

    DAC_Type *dac;
    
    DRIVER_ASSERT(id < DAC_ID_MAX);
    
    dac = DAC_REG(id);

    dac->CR |= (0x01UL << DAC_CR_DAC_EN_Pos);

    /* 20us delay */
    Count = (SystemCoreClock / 1000000) * 2;
    if (Count < 5)
    {
        Count = 5;
    }
    for (i = 0; i < Count; i++)
    {
        __NOP();
    }

    dac->CR |= (0x01UL << DAC_CR_DAC_BUFEN_Pos);
    
    return DRIVER_ERROR_OK;
}

int32_t HAL_DAC_Stop(enum dac_id id)
{
    DAC_Type *dac;
    
    DRIVER_ASSERT(id < DAC_ID_MAX);
    
    dac = DAC_REG(id);
    dac->CR &= ~(0x01UL << DAC_CR_DAC_BUFEN_Pos);
    dac->CR &= ~(0x01UL << DAC_CR_DAC_EN_Pos);
    
    return DRIVER_ERROR_OK;
}

int32_t HAL_DAC_SetData(enum dac_id id, uint32_t data)
{
    DAC_Type *dac;
    
    DRIVER_ASSERT(id < DAC_ID_MAX);
    
    dac = DAC_REG(id);
    dac->DR = (data & 0xFFF);    
    
    return DRIVER_ERROR_OK;
}

int32_t HAL_DAC_GetData(enum dac_id id, uint32_t *data)
{
    DAC_Type *dac;

    DRIVER_ASSERT(id < DAC_ID_MAX);

    dac = DAC_REG(id);
    *data = dac->DR;

    return DRIVER_ERROR_OK;
}
