/**
 *******************************************************************************
 * @file        a34m456_gpio.c
 * @author      ABOV R&D Division
 * @brief       gpio driver
 *
 * Copyright 2024 ABOV Semiconductor Co.,Ltd. All rights reserved.
 *
 * This file is licensed under terms that are found in the LICENSE file
 * located at Document directory.
 * If this file is delivered or shared without applicable license terms,
 * the terms of the BSD-3-Clause license shall be applied.
 * Reference: https://opensource.org/licenses/BSD-3-Clause
 ******************************************************************************/

#include <stdio.h>
#include "a34m456_gpio.h"
#include "a34m456_scu.h"


#define PIN_OFFSET(pin)                 (pin << 1)

#define PCU_MR_FUNC_Msk               (0x07UL)
#define PCU_CR_IO_MODE_Msk              (0x03UL)
#define PCU_CR_OUTPUT_MODE_Msk          (0x01UL)
#define PCU_PRCR_Msk                  (0x03UL)
#define PCU_STR_Msk                 (0x01UL)
#define PCU_IER_Msk                     (0x03UL)
#define PCU_ICR_Msk                     (0x03UL)

#define GPIO_REG_OFFSET                 (0x100UL)

static __INLINE PCU_Type *GPIO_REG(enum gpio_port port)
{
    return (PCU_Type *)(PA_BASE + (GPIO_REG_OFFSET * port));
}

static __INLINE IRQn_Type GPIO_IRQ_NUM(enum gpio_port port)
{
    return (IRQn_Type)((uint8_t)GPIOA_IRQn + port);
}

struct gpio_ctrl_block
{
    enum gpio_port      port;
    enum gpio_pin       pin;
    gpio_irq_handler_t  handler;
    void *              context;
};

static struct gpio_ctrl_block g_gcb[CONFIG_GPIO_NUM_IRQ];

static uint8_t gpio_irq_setup[PORT_MAX];

static int8_t GPIO_EmptySlot(enum gpio_port port, enum gpio_pin pin)
{
    uint8_t i;

    for (i = 0; i < CONFIG_GPIO_NUM_IRQ; i++)
    {
        if (g_gcb[i].port == port && g_gcb[i].pin == pin)
        {
            return i;
        }

        if (g_gcb[i].handler == NULL)
        {
            return i;
        }
    }
    return -1;
}

static void GPIO_SetupIRQ(enum gpio_port port, uint8_t irq_prio)
{
    if (gpio_irq_setup[port])
    {
        return;
    }

    NVIC_ClearPendingIRQ(GPIO_IRQ_NUM(port));
    NVIC_SetPriority(GPIO_IRQ_NUM(port), irq_prio);
    NVIC_EnableIRQ(GPIO_IRQ_NUM(port));

    gpio_irq_setup[port] = 1;
}

int32_t HAL_GPIO_Init(void)
{

    SCU->PER1 |= 0x00UL
            | (0x01UL << SCU_PER1_GPIOF_Pos)
            | (0x01UL << SCU_PER1_GPIOE_Pos)
            | (0x01UL << SCU_PER1_GPIOD_Pos)
            | (0x01UL << SCU_PER1_GPIOC_Pos)
            | (0x01UL << SCU_PER1_GPIOB_Pos)
            | (0x01UL << SCU_PER1_GPIOA_Pos);

    SCU->PCER1 |= 0x00UL
            | (0x01UL << SCU_PCER1_GPIOF_Pos)
            | (0x01UL << SCU_PCER1_GPIOE_Pos)
            | (0x01UL << SCU_PCER1_GPIOD_Pos)
            | (0x01UL << SCU_PCER1_GPIOC_Pos)
            | (0x01UL << SCU_PCER1_GPIOB_Pos)
            | (0x01UL << SCU_PCER1_GPIOA_Pos);

    return DRIVER_ERROR_OK;
}

int32_t HAL_GPIO_SetMode(enum gpio_port port, enum gpio_pin pin, enum gpio_mode mode)
{
    PCU_Type *pcu;
    uint32_t type;
    uint32_t reg_val;

    DRIVER_ASSERT(port < PORT_MAX && pin < PIN_MAX && mode < GPIO_MODE_MAX);

    pcu = GPIO_REG(port);

    if (mode == GPIO_MODE_OUTPUT)
    {
        /* default push pull type */
        type = 0x00;
    }
    else
    {
        type = 0x03;
    }

    reg_val = pcu->CR;
    reg_val &= ~(PCU_CR_IO_MODE_Msk << (pin << 1));
    reg_val |= (type << (pin << 1));

    pcu->CR = reg_val;

    return DRIVER_ERROR_OK;
}

int32_t HAL_GPIO_SetFunction(enum gpio_port port, enum gpio_pin pin, uint32_t func)
{
    PCU_Type *pcu;
    uint32_t pin_offset;
    uint32_t reg_val;

    pcu = GPIO_REG(port);

    if (pin > 7)
    {
        pin_offset = ((pin - 8) << 2);

        reg_val = pcu->MR2;
        reg_val &= ~(PCU_MR_FUNC_Msk << pin_offset);
        reg_val |= (func << pin_offset);

        pcu->MR2 = reg_val;
    }
    else
    {
        pin_offset = (pin << 2);

        reg_val = pcu->MR1;
        reg_val &= ~(PCU_MR_FUNC_Msk << pin_offset);
        reg_val |= (func << pin_offset);

        pcu->MR1 = reg_val;
    }

    return DRIVER_ERROR_OK;
}

int32_t HAL_GPIO_SetOutput(enum gpio_port port, enum gpio_pin pin, enum gpio_output_type type)
{
    PCU_Type *pcu;
    uint32_t reg_val;

    DRIVER_ASSERT(port < PORT_MAX && pin < PIN_MAX);

    pcu = GPIO_REG(port);

    reg_val = pcu->CR;
    reg_val &= ~(PCU_CR_OUTPUT_MODE_Msk << (pin << 1));
    reg_val |= (type << (pin << 1));

    pcu->CR = reg_val;

    return DRIVER_ERROR_OK;
}

int32_t HAL_GPIO_SetPull(enum gpio_port port, enum gpio_pin pin, enum gpio_pull pull)
{
    PCU_Type *pcu;
    uint32_t reg_val;

    DRIVER_ASSERT(port < PORT_MAX && pin < PIN_MAX);

    pcu = GPIO_REG(port);

    reg_val = pcu->PRCR;
    reg_val &= ~(PCU_PRCR_Msk << (pin << 1));
    reg_val |= (pull << (pin << 1));

    pcu->PRCR = reg_val;

    return DRIVER_ERROR_OK;
}

int32_t HAL_GPIO_SetStrength(enum gpio_port port, enum gpio_pin pin, enum gpio_strength strength)
{
    PCU_Type *pcu;
    uint32_t reg_val;

    DRIVER_ASSERT(port < PORT_MAX && pin < PIN_MAX);

    pcu = GPIO_REG(port);

    reg_val = pcu->STR;
    reg_val &= ~(PCU_STR_Msk << pin);
    reg_val |= (strength << pin);
    pcu->STR = reg_val;

    return DRIVER_ERROR_OK;
}

int32_t HAL_GPIO_SetDebounce(enum gpio_port port, enum gpio_pin pin, uint8_t enable)
{
    PCU_Type *pcu;
    uint32_t reg_val;

    DRIVER_ASSERT(port < PORT_MAX && pin < PIN_MAX);

    pcu = GPIO_REG(port);

    reg_val = pcu->DER;
    reg_val &= ~(1 << pin);
    if (enable)
    {
        reg_val |= (1 << pin);
    }
    pcu->DER = reg_val;

    return DRIVER_ERROR_OK;
}

int32_t HAL_GPIO_SetDebounceClock(enum gpio_port port, enum gpio_dbc_clk_src clk_src,
                                  uint8_t clk_div)
{
    uint32_t clk_val = 0;

    DRIVER_ASSERT(port < PORT_MAX);
    DRIVER_ASSERT(clk_div > 0);

    switch (port)
    {
        case PORTA:
        case PORTD:
        case PORTE:
        case PORTF:
            clk_val = SCU->MCCR4;
            clk_val &= ~(SCU_MCCR4_PGADCSEL_Msk | SCU_MCCR4_PGADCDIV_Msk);
            SCU->MCCR4 = clk_val | (clk_src << SCU_MCCR4_PGADCSEL_Pos) | (clk_div << SCU_MCCR4_PGADCDIV_Pos);
            break;
        case PORTB:
            clk_val = SCU->MCCR5;
            clk_val &= ~(SCU_MCCR5_PGBDCSEL_Msk | SCU_MCCR5_PGBDCDIV_Msk);
            SCU->MCCR5 = clk_val | (clk_src << SCU_MCCR5_PGBDCSEL_Pos) | (clk_div << SCU_MCCR5_PGBDCDIV_Pos);
            break;
        case PORTC:
            clk_val = SCU->MCCR5;
            clk_val &= ~(SCU_MCCR5_PGCDCSEL_Msk | SCU_MCCR5_PGCDCDIV_Msk);
            SCU->MCCR5 = clk_val | (clk_src << SCU_MCCR5_PGCDCSEL_Pos) | (clk_div << SCU_MCCR5_PGCDCDIV_Pos);
            break;
        default:
            break;
    }

    return DRIVER_ERROR_OK;
}

int32_t HAL_GPIO_EnableIRQ(enum gpio_port port, enum gpio_pin pin,
                           enum gpio_irq_trig trig,
                           enum gpio_irq_mode mode,
                           uint8_t irq_prio)
{
    PCU_Type *pcu;
    uint32_t ier, icr;

    DRIVER_ASSERT(port < PORT_MAX && pin < PIN_MAX);

    pcu = GPIO_REG(port);

    ier = pcu->IER;
    icr = pcu->ICR;

    ier &= ~(PCU_IER_Msk << PIN_OFFSET(pin));
    icr &= ~(PCU_ICR_Msk << PIN_OFFSET(pin));

    ier |= (trig << PIN_OFFSET(pin));
    icr |= (mode << PIN_OFFSET(pin));

    pcu->ISR = (0x03 << PIN_OFFSET(pin)); // Clear interrupt flag

    pcu->IER = ier;
    pcu->ICR = icr;

    GPIO_SetupIRQ(port, irq_prio);

    return DRIVER_ERROR_OK;
}

int32_t HAL_GPIO_DisableIRQ(enum gpio_port port, enum gpio_pin pin)
{
    PCU_Type *pcu;
    uint32_t ier, icr;

    DRIVER_ASSERT(port < PORT_MAX && pin < PIN_MAX);

    pcu = GPIO_REG(port);

    ier = pcu->IER;
    icr = pcu->ICR;

    ier &= ~(0x03 << PIN_OFFSET(pin));
    icr &= ~(0x03 << PIN_OFFSET(pin));

    pcu->IER = ier;
    pcu->ICR = icr;

    pcu->ISR = (0x03 << PIN_OFFSET(pin)); // Clear interrupt flag

    if (!ier)
    {
        NVIC_DisableIRQ(GPIO_IRQ_NUM(port));
		gpio_irq_setup[port] = 0;
    }

    return DRIVER_ERROR_OK;
}

int32_t HAL_GPIO_SetIRQHandler(enum gpio_port port, enum gpio_pin pin, gpio_irq_handler_t handler, void *context)
{
    int8_t i;

    DRIVER_ASSERT(port < PORT_MAX && pin < PIN_MAX);

    i = GPIO_EmptySlot(port, pin);
    if (i == -1)
    {
        return DRIVER_ERROR_NO_RESOURCE;
    }

    g_gcb[i].port = port;
    g_gcb[i].pin = pin;
    g_gcb[i].handler = handler;
    g_gcb[i].context = context;

    return DRIVER_ERROR_OK;
}

int32_t HAL_GPIO_SetPin(enum gpio_port port, enum gpio_pin pin)
{
    PCU_Type *pcu;

    DRIVER_ASSERT(port < PORT_MAX && pin < PIN_MAX);

    pcu = GPIO_REG(port);

    pcu->BSR = (1 << pin);

    return DRIVER_ERROR_OK;
}

int32_t HAL_GPIO_ClearPin(enum gpio_port port, enum gpio_pin pin)
{
    PCU_Type *pcu;

    DRIVER_ASSERT(port < PORT_MAX && pin < PIN_MAX);

    pcu = GPIO_REG(port);

    pcu->BCR = (1 << pin);

    return DRIVER_ERROR_OK;
}

int32_t HAL_GPIO_ReadPin(enum gpio_port port, enum gpio_pin pin, uint32_t *value)
{
    PCU_Type *pcu;
    uint32_t reg_val;

    DRIVER_ASSERT(port < PORT_MAX && pin < PIN_MAX);

    pcu = GPIO_REG(port);

    reg_val = pcu->IDR;

    if ((reg_val >> (pin)) & 0x01)
    {
        *value = 1;
    }
    else
    {
        *value = 0;
    }

    return DRIVER_ERROR_OK;
}

int32_t HAL_GPIO_WritePort(enum gpio_port port, uint32_t value)
{
    PCU_Type *pcu;

    DRIVER_ASSERT(port < PORT_MAX);

    pcu = GPIO_REG(port);

    pcu->ODR = value;

    return DRIVER_ERROR_OK;
}

int32_t HAL_GPIO_ReadPort(enum gpio_port port, uint32_t *value)
{
    PCU_Type *pcu;

    DRIVER_ASSERT(port < PORT_MAX);

    pcu = GPIO_REG(port);

    *value = pcu->IDR;

    return DRIVER_ERROR_OK;
}


static void GPIO_Handler(enum gpio_port port)
{
    PCU_Type *pcu;
    uint32_t isr;
    uint32_t pin;
    uint32_t i;
    struct gpio_ctrl_block *gcb;

    pcu = GPIO_REG(port);
    isr = pcu->ISR;

    for (pin = 0; pin < PIN_MAX; pin++)
    {
        if ((isr >> (pin << 1)) & 0x03)
        {
            for (i = 0; i < CONFIG_GPIO_NUM_IRQ; i++)
            {
                gcb = &g_gcb[i];
                if (gcb->port == port && gcb->pin == pin)
                {
                    if (gcb->handler)
                    {
                        gcb->handler(gcb->context);
                    }
                    break;
                }
            }
        }
    }
    pcu->ISR = isr;
}

void GPIOA_IRQHandler(void)
{
    GPIO_Handler(PORTA);
}

void GPIOB_IRQHandler(void)
{
    GPIO_Handler(PORTB);
}

void GPIOC_IRQHandler(void)
{
    GPIO_Handler(PORTC);
}

void GPIOD_IRQHandler(void)
{
    GPIO_Handler(PORTD);
}

void GPIOE_IRQHandler(void)
{
    GPIO_Handler(PORTE);
}

void GPIOF_IRQHandler(void)
{
    GPIO_Handler(PORTF);
}
