/**
 *******************************************************************************
 * @file        a34m456_pfcp.c
 * @author      ABOV R&D Division
 * @brief       power factor correction pwm
 *
 * Copyright 2024 ABOV Semiconductor Co.,Ltd. All rights reserved.
 *
 * This file is licensed under terms that are found in the LICENSE file
 * located at Document directory.
 * If this file is delivered or shared without applicable license terms,
 * the terms of the BSD-3-Clause license shall be applied.
 * Reference: https://opensource.org/licenses/BSD-3-Clause
 ******************************************************************************/

#include <stdint.h>
#include <stdlib.h>
#include <string.h>

#include "a34m456.h"
#include "a34m456_scu.h"
#include "a34m456_pfcp.h"

#define PFCP_FOLR_FOLKEY                   (0xCAUL)

#define PFCP_REG_BIT_SET(r, b, pos)        ((b == 1) ? (r |= (1UL << pos)) : (r &= ~(1UL << pos)))
#define PFCP_REG_BIT_SET2(r, b, b2, pos)   (((b == 1) || (b2 == 1)) ? (r |= (1UL << pos)) : (r &= ~(1UL << pos)))
#define PFCP_REG_VAL_SET(r, d, pos, msk)   r &= ~msk; r |= ((d << pos) & msk)
#define PFCP_REG_OFFSET         0x0UL

struct pfcp_ctrl_block
{
    uint8_t             mode;
    pfcp_irq_handler_t  handler;
    void *              context;
};

static struct pfcp_ctrl_block g_mcb[PFCP_ID_MAX];

static __INLINE PFCP_Type *PFCP_REG(enum pfcp_id id)
{
    return (PFCP_Type *)(PFCP_BASE + (PFCP_REG_OFFSET * id));
}

static __INLINE IRQn_Type PFCP_PROT0_IRQ_NUM(enum pfcp_id id)
{
    return (IRQn_Type)((uint8_t)PFCPPROT0_IRQn + id);
}
static __INLINE IRQn_Type PFCP_PROT1_IRQ_NUM(enum pfcp_id id)
{
    return (IRQn_Type)((uint8_t)PFCPPROT1_IRQn + id);
}
static __INLINE IRQn_Type PFCP_IRQ_NUM(enum pfcp_id id)
{
    return (IRQn_Type)((uint8_t)PFCP_IRQn + id);
}

static void PFCP_Configure(PFCP_Type *pfcp, struct pfcp_cfg *cfg)
{
    uint32_t reg_val = 0;

    reg_val =((cfg->clk_div << PFCP_MR_CLKDIV_Pos) & PFCP_MR_CLKDIV_Msk) |
               (cfg->mode << PFCP_MR_PFCPMOD_Pos) |
               (cfg->count_mode << PFCP_MR_UPDOWN_Pos) |
               (cfg->asym_mode << PFCP_MR_ASYM_Pos) |
               (cfg->out_lv_upd_ao_ab << PFCP_MR_UABOL_Pos) |
               (cfg->out_lv_upd_top_b << PFCP_MR_TUOL_B_Pos) |
               (cfg->out_lv_upd_btm_b << PFCP_MR_BUOL_B_Pos) |
               (cfg->out_lv_upd_top_a << PFCP_MR_TUOL_A_Pos) |
               (cfg->out_lv_upd_btm_a << PFCP_MR_BUOL_A_Pos) |
               (cfg->d_upd_ao_ab << PFCP_MR_UAB_Pos) |
               (cfg->d_upd_top_b << PFCP_MR_TUP_B_Pos) |
               (cfg->d_upd_btm_b << PFCP_MR_BUP_B_Pos) |
               (cfg->d_upd_top_a << PFCP_MR_TUP_A_Pos) |
               (cfg->d_upd_btm_a << PFCP_MR_BUP_A_Pos) |
               (cfg->p_upd_top << PFCP_MR_PTUP_Pos)	|
               (cfg->p_upd_btm << PFCP_MR_PBUP_Pos);

    pfcp->MR = reg_val;
}

int32_t HAL_PFCP_Init(enum pfcp_id id, struct pfcp_cfg *cfg,
                      pfcp_irq_handler_t handler,
                      void *context)
{
    struct pfcp_ctrl_block *mcb;
    PFCP_Type *pfcp;

    DRIVER_ASSERT(id < PFCP_ID_MAX);
    DRIVER_ASSERT(cfg != NULL);

    mcb = &g_mcb[id];
    pfcp = PFCP_REG(id);

    memset(mcb, 0, sizeof(struct pfcp_ctrl_block));
    if(id==PFCP_ID_0)
    {
        SCU->PER2 &= (~SCU_PER2_PFCP_Msk);
        SCU->PCER2 &= (~SCU_PCER2_PFCP_Msk);
        SCU->PER2 |= (1 << SCU_PER2_PFCP_Pos);
        SCU->PCER2 |= (1 << SCU_PCER2_PFCP_Pos);
    }
    else
    {
/*
        SCU->PER2 &= (~SCU_PER2_PFCP_Msk);
        SCU->PCER2 &= (~SCU_PCER2_PFCP_Msk);
        SCU->PER2 |= (1 << SCU_PER2_PFCP_Pos);
        SCU->PCER2 |= (1 << SCU_PCER2_PFCP_Pos);
*/		
    }
    mcb->handler = handler;
    mcb->context = context;

    PFCP_REG_BIT_SET(pfcp->CR, 1, PFCP_CR_PWMEN_Pos);

    HAL_PFCP_DisablePWMOutput(id);

    PFCP_Configure(pfcp, cfg);

    return DRIVER_ERROR_OK;
}

int32_t HAL_PFCP_Deinit(enum pfcp_id id)
{
    DRIVER_ASSERT(id <  PFCP_ID_MAX);
    if(id==PFCP_ID_0)
    {
        SCU->PER2 &= (~SCU_PER2_PFCP_Msk);
        SCU->PCER2 &= (~SCU_PCER2_PFCP_Msk);
    }
    else
    {
        SCU->PER2 &= (~SCU_PER2_PFCP_Msk);
        SCU->PCER2 &= (~SCU_PCER2_PFCP_Msk);
    }


    NVIC_DisableIRQ(PFCP_PROT0_IRQ_NUM(id));
    NVIC_DisableIRQ(PFCP_PROT1_IRQ_NUM(id));
    NVIC_DisableIRQ(PFCP_IRQ_NUM(id));

    return DRIVER_ERROR_OK;
}

int32_t HAL_PFCP_SetIRQConfig(enum pfcp_id id, struct pfcp_irq_cfg *cfg)
{
    uint32_t reg_val = 0;
    PFCP_Type *pfcp;

    DRIVER_ASSERT(id < PFCP_ID_MAX);
    pfcp = PFCP_REG(id);

    reg_val  =((cfg->prdie_b << PFCP_IER_PRDIE_B_Pos) |
               (cfg->botie_b << PFCP_IER_BOTIE_B_Pos) |
			   (cfg->prdie_a << PFCP_IER_PRDIE_A_Pos) |
               (cfg->botie_a << PFCP_IER_BOTIE_A_Pos) |
               (cfg->atr6ie << PFCP_IER_ATR6IE_Pos) |
               (cfg->atr5ie << PFCP_IER_ATR5IE_Pos) |
               (cfg->atr4ie << PFCP_IER_BHIE_ATR4IE_Pos) |
               (cfg->atr3ie << PFCP_IER_BLIE_ATR3IE_Pos) |
               (cfg->atr2ie << PFCP_IER_AHIE_ATR2IE_Pos) |
               (cfg->atr1ie << PFCP_IER_ALIE_ATR1IE_Pos));
    pfcp->IER = reg_val;

    if (reg_val)
    {
        NVIC_ClearPendingIRQ(PFCP_IRQ_NUM(id));
        NVIC_SetPriority(PFCP_IRQ_NUM(id), cfg->irq_prio);
        NVIC_EnableIRQ(PFCP_IRQ_NUM(id));
    }
    else
    {
        NVIC_DisableIRQ(PFCP_IRQ_NUM(id));
    }
    return DRIVER_ERROR_OK;
}

int32_t HAL_PFCP_SetProtection0InputConfig(enum pfcp_id id, pfcp_irq_handler_t handler,
                                         struct pfcp_input_prt0_cfg *cfg)
{
    uint32_t reg_val;
    PFCP_Type *pfcp;

    DRIVER_ASSERT(id < PFCP_ID_MAX);

    pfcp = PFCP_REG(id);

    reg_val = (((cfg->polarity << PFCP_PCR0_PROTPOL_Pos) & PFCP_PCR0_PROTPOL_Msk) |
               ((cfg->clk_div << PFCP_PCR0_PROTDIV_Pos) & PFCP_PCR0_PROTDIV_Msk) |
               ((cfg->debounce << PFCP_PCR0_PROTD_Pos) & PFCP_PCR0_PROTD_Msk) |
               ((cfg->enable_prt7 << PFCP_PCR0_PROT7EN_Pos) & PFCP_PCR0_PROT7EN_Msk) |
               ((cfg->enable_prt6 << PFCP_PCR0_PROT6EN_Pos) & PFCP_PCR0_PROT6EN_Msk) |
               ((cfg->enable_prt5 << PFCP_PCR0_PROT5EN_Pos) & PFCP_PCR0_PROT5EN_Msk) |
               ((cfg->enable_prt4 << PFCP_PCR0_PROT4EN_Pos) & PFCP_PCR0_PROT4EN_Msk) |
               ((cfg->enable_prt3 << PFCP_PCR0_PROT3EN_Pos) & PFCP_PCR0_PROT3EN_Msk) |
               ((cfg->enable_prt2 << PFCP_PCR0_PROT2EN_Pos) & PFCP_PCR0_PROT2EN_Msk) |
               ((cfg->enable_prt1 << PFCP_PCR0_PROT1EN_Pos) & PFCP_PCR0_PROT1EN_Msk) |
               ((cfg->enable_prt0 << PFCP_PCR0_PROT0EN_Pos) & PFCP_PCR0_PROT0EN_Msk));
    if(reg_val)
    {
        /* Protection interrupt  */
        NVIC_ClearPendingIRQ(PFCP_PROT0_IRQ_NUM(id));
        NVIC_SetPriority(PFCP_PROT0_IRQ_NUM(id), DRIVER_PRIORITY_1);
        NVIC_EnableIRQ(PFCP_PROT0_IRQ_NUM(id));
    }
    pfcp->PCR0 = reg_val;

    return DRIVER_ERROR_OK;
}

int32_t HAL_PFCP_SetProtection0OutputConfig(enum pfcp_id id, pfcp_irq_handler_t handler,
                                           enum pfcp_output_type ot, uint32_t en)
{
    return DRIVER_ERROR_NOT_SUPPORTED;
}

int32_t HAL_PFCP_SetProtection1InputConfig(enum pfcp_id id, pfcp_irq_handler_t handler,
                                           struct pfcp_input_prt1_cfg *cfg)
{
    uint32_t reg_val;
    PFCP_Type *pfcp;

    DRIVER_ASSERT(id < PFCP_ID_MAX);

    pfcp = PFCP_REG(id);

    reg_val = (((cfg->polarity << PFCP_PCR1_PROTPOL_Pos) & PFCP_PCR1_PROTPOL_Msk) |
               ((cfg->clk_div << PFCP_PCR1_PROTDIV_Pos) & PFCP_PCR1_PROTDIV_Msk) |
               ((cfg->debounce << PFCP_PCR1_PROTD_Pos) & PFCP_PCR1_PROTD_Msk) |
               ((cfg->enable_prt7 << PFCP_PCR1_PROT7EN_Pos) & PFCP_PCR1_PROT7EN_Msk) |
               ((cfg->enable_prt6 << PFCP_PCR1_PROT6EN_Pos) & PFCP_PCR1_PROT6EN_Msk) |
               ((cfg->enable_prt5 << PFCP_PCR1_PROT5EN_Pos) & PFCP_PCR1_PROT5EN_Msk) |
               ((cfg->enable_prt4 << PFCP_PCR1_PROT4EN_Pos) & PFCP_PCR1_PROT4EN_Msk) |
               ((cfg->enable_prt3 << PFCP_PCR1_PROT3EN_Pos) & PFCP_PCR1_PROT3EN_Msk) |
               ((cfg->enable_prt2 << PFCP_PCR1_PROT2EN_Pos) & PFCP_PCR1_PROT2EN_Msk) |
               ((cfg->enable_prt1 << PFCP_PCR1_PROT1EN_Pos) & PFCP_PCR1_PROT1EN_Msk) |
               ((cfg->enable_prt0 << PFCP_PCR1_PROT0EN_Pos) & PFCP_PCR1_PROT0EN_Msk));

    if(reg_val)
    {
        /* Protection interrupt  */
        NVIC_ClearPendingIRQ(PFCP_PROT1_IRQ_NUM(id));
        NVIC_SetPriority(PFCP_PROT1_IRQ_NUM(id), DRIVER_PRIORITY_1);
        NVIC_EnableIRQ(PFCP_PROT1_IRQ_NUM(id));
    }
    pfcp->PCR1 = reg_val;

    return DRIVER_ERROR_OK;
}

int32_t HAL_PFCP_SetPotection1OutputConfig(enum pfcp_id id, pfcp_irq_handler_t handler,
                                            enum pfcp_output_type ot, uint32_t en)
{
    return DRIVER_ERROR_NOT_SUPPORTED;
}

int32_t HAL_PFCP_SetDeadTimeConfig(enum pfcp_id id, enum pfcp_phase_type p_type, struct pfcp_dt_cfg *cfg)
{
    uint32_t reg_val;
    PFCP_Type *pfcp;

    DRIVER_ASSERT(id < PFCP_ID_MAX);

    pfcp = PFCP_REG(id);

    reg_val = (((cfg->enable << PFCP_DTR_DTEN_Pos) & PFCP_DTR_DTEN_Msk) |
               ((cfg->short_circuit_protect_en << PFCP_DTR_PSHRT_Pos) & PFCP_DTR_PSHRT_Msk) |
               ((cfg->clk << PFCP_DTR_DTCLK_Pos) & PFCP_DTR_DTCLK_Msk) |
               ((cfg->output_level_bside << PFCP_DTR_BDTOL_Pos) & PFCP_DTR_BDTOL_Msk) |
               ((cfg->output_level_aside << PFCP_DTR_ADTOL_Pos) & PFCP_DTR_ADTOL_Msk) |
               ((cfg->bside_dead_time << PFCP_DTR_BDT_Pos) & PFCP_DTR_BDT_Msk) |
               ((cfg->aside_dead_time << PFCP_DTR_ADT_Pos) & PFCP_DTR_ADT_Msk));
    pfcp->DTR = reg_val;

    return DRIVER_ERROR_OK;
}

int32_t HAL_PFCP_SetADCTriggerConfig(enum pfcp_id id, enum pfcp_trgger_id t_id, struct pfcp_adc_tr_cfg *cfg)
{
    uint32_t reg_val;
    PFCP_Type *pfcp;

    DRIVER_ASSERT(id < PFCP_ID_MAX);

    pfcp = PFCP_REG(id);

    reg_val = (((cfg->reg_mode << PFCP_ATR1_ATUDT_Pos) & PFCP_ATR1_ATUDT_Msk) |
               ((cfg->mode << PFCP_ATR1_ATMOD_Pos) & PFCP_ATR1_ATMOD_Msk) |
               ((cfg->tr_count << PFCP_ATR1_ATCNT_Pos) & PFCP_ATR1_ATCNT_Msk));
    switch (t_id)
    {
        case PFCP_TRGGER_ID_1:
            pfcp->ATR1 = reg_val;
            break;
        case PFCP_TRGGER_ID_2:
            pfcp->ATR2 = reg_val;
            break;
        case PFCP_TRGGER_ID_3:
            pfcp->ATR3 = reg_val;
            break;
        case PFCP_TRGGER_ID_4:
            pfcp->ATR4 = reg_val;
            break;
        case PFCP_TRGGER_ID_5:
            pfcp->ATR5 = reg_val;
            break;
        case PFCP_TRGGER_ID_6:
            pfcp->ATR6 = reg_val;
            break;
    }


    return DRIVER_ERROR_OK;
}

int32_t HAL_PFCP_SetIRQinterval(enum pfcp_id id, enum pfcp_phase_type p_type,
                                uint32_t irq_intv)
{
    uint32_t reg_val;
    PFCP_Type *pfcp;

    DRIVER_ASSERT(id < PFCP_ID_MAX);

    pfcp = PFCP_REG(id);

    reg_val = pfcp->CR;
    PFCP_REG_VAL_SET(reg_val, irq_intv, PFCP_CR_IRQN_Pos, PFCP_CR_IRQN_Msk);
    pfcp->CR = reg_val;

    return DRIVER_ERROR_OK;
}

int32_t HAL_PFCP_ControlPWMOutput(enum pfcp_id id,
                                  enum pfcp_output_type ot, uint32_t en)
{
    uint32_t reg_val;
    PFCP_Type *pfcp;

    DRIVER_ASSERT(id < PFCP_ID_MAX);

    pfcp = PFCP_REG(id);

    reg_val = pfcp->OLR;
    if (ot == PFCP_OUTPUT_TYPE_A)
    {
        PFCP_REG_BIT_SET(reg_val, en, PFCP_OLR_AOC_Pos);
    }
    else
    {
        PFCP_REG_BIT_SET(reg_val, en, PFCP_OLR_BOC_Pos);
    }

    PFCP->OLR = reg_val;

    return DRIVER_ERROR_OK;
}

int32_t HAL_PFCP_SetDisabledOutputLevel(enum pfcp_id id,
                                        enum pfcp_output_type ot, uint32_t hl)
{
    uint32_t reg_val;
    PFCP_Type *pfcp;

    DRIVER_ASSERT(id < PFCP_ID_MAX);

    pfcp = PFCP_REG(id);

    reg_val = pfcp->OLR;
    if (ot == PFCP_OUTPUT_TYPE_A)
    {
        PFCP_REG_BIT_SET(reg_val, hl, PFCP_OLR_ADOL_Pos);
    }
    else
    {
        PFCP_REG_BIT_SET(reg_val, hl, PFCP_OLR_BDOL_Pos);
    }
    pfcp->OLR = reg_val;

    return DRIVER_ERROR_OK;
}

int32_t HAL_PFCP_SetStartOutputLevel(enum pfcp_id id,
                                     enum pfcp_output_type ot, uint32_t hl)
{
    uint32_t reg_val;
    PFCP_Type *pfcp;

    DRIVER_ASSERT(id < PFCP_ID_MAX);

    pfcp = PFCP_REG(id);

    reg_val = pfcp->OLR;
    if (ot == PFCP_OUTPUT_TYPE_A)
    {
        PFCP_REG_BIT_SET(reg_val, hl, PFCP_OLR_ASL_Pos);
    }
    else
    {
        PFCP_REG_BIT_SET(reg_val, hl, PFCP_OLR_BSL_Pos);
    }

    pfcp->OLR = reg_val;

    return DRIVER_ERROR_OK;
}

int32_t HAL_PFCP_SetForceOutputLevel(enum pfcp_id id,
                                     enum pfcp_output_type ot, uint8_t hl)
{
    uint32_t reg_val;
    PFCP_Type *pfcp;

    DRIVER_ASSERT(id < PFCP_ID_MAX);

    pfcp = PFCP_REG(id);

    reg_val = pfcp->FOLR;
    if (ot == PFCP_OUTPUT_TYPE_A)
    {
        PFCP_REG_BIT_SET(reg_val, hl, PFCP_FOLR_AFL_Pos);
    }
    else
    {
        PFCP_REG_BIT_SET(reg_val, hl, PFCP_FOLR_BFL_Pos);
    }

    pfcp->FOLR = reg_val;

    return DRIVER_ERROR_OK;
}

int32_t HAL_PFCP_SetPeriod(enum pfcp_id id, uint16_t prd)
{
    PFCP_Type *pfcp;

    DRIVER_ASSERT(id < PFCP_ID_MAX);

    pfcp = PFCP_REG(id);

    if (prd < 0x10)
    {
        return DRIVER_ERROR_PARAMETER;
    }

    pfcp->PRD = prd;

    return DRIVER_ERROR_OK;
}

int32_t HAL_PFCP_SetDuty(enum pfcp_id id, enum pfcp_output_type ot,
                        enum pfcp_duty_type d_type, uint16_t duty)
{
    PFCP_Type *pfcp;

    DRIVER_ASSERT(id < PFCP_ID_MAX);

    pfcp = PFCP_REG(id);

    if (ot == PFCP_OUTPUT_TYPE_A )
    {
        if (d_type == PFCP_DUTY_TYPE_L)
        {
            pfcp->DAL = duty;
        }
        else
        {
            pfcp->DAH = duty;
        }
    }

    if (ot == PFCP_OUTPUT_TYPE_B )
    {
        if (d_type == PFCP_DUTY_TYPE_L)
        {
            pfcp->DBL = duty;
        }
        else
        {
            pfcp->DBH = duty;
        }
    }

    return DRIVER_ERROR_OK;
}

int32_t HAL_PFCP_Start(enum pfcp_id id, enum pfcp_phase_type p_type)
{
    PFCP_Type *pfcp;

    DRIVER_ASSERT(id < PFCP_ID_MAX);

    pfcp = PFCP_REG(id);

    pfcp->CR |= (0x01UL << PFCP_CR_PSTART_Pos);

    return DRIVER_ERROR_OK;
}

int32_t HAL_PFCP_Stop(enum pfcp_id id, enum pfcp_phase_type p_type)
{
    PFCP_Type *pfcp;

    DRIVER_ASSERT(id < PFCP_ID_MAX);

    pfcp = PFCP_REG(id);

    pfcp->CR &= ~(0x01UL << PFCP_CR_PSTART_Pos);

    return DRIVER_ERROR_OK;
}


int32_t HAL_PFCP_ReadCount(enum pfcp_id id, enum pfcp_output_type ot, uint16_t *count)
{
    PFCP_Type *pfcp;

    DRIVER_ASSERT(id < PFCP_ID_MAX);

    pfcp = PFCP_REG(id);
	
    if (ot == PFCP_OUTPUT_TYPE_A)
    {
        *count = pfcp->ACNT;
    }
    else
    {
        *count = pfcp->BCNT;
    }

    return DRIVER_ERROR_OK;
}

int32_t HAL_PFCP_ReadStatus(enum pfcp_id id, uint32_t *status)
{
    PFCP_Type *pfcp;

    DRIVER_ASSERT(id < PFCP_ID_MAX);

    pfcp = PFCP_REG(id);

    *status = pfcp->SR;

    return DRIVER_ERROR_OK;
}

int32_t HAL_PFCP_ReadProtection0Status(enum pfcp_id id, uint32_t *status)
{
    PFCP_Type *pfcp;

    DRIVER_ASSERT(id < PFCP_ID_MAX);

    pfcp = PFCP_REG(id);

    *status = pfcp->PSR0;

    return DRIVER_ERROR_OK;
}

int32_t HAL_PFCP_ReadProtection1Status(enum pfcp_id id, uint32_t *status)
{
    PFCP_Type *pfcp;

    DRIVER_ASSERT(id < PFCP_ID_MAX);

    pfcp = PFCP_REG(id);

    *status = pfcp->PSR1;

    return DRIVER_ERROR_OK;
}

int32_t HAL_PFCP_EnablePWMOutput(enum pfcp_id id)
{
    uint32_t reg_val;
    PFCP_Type *pfcp;

    DRIVER_ASSERT(id < PFCP_ID_MAX);

    pfcp = PFCP_REG(id);

    reg_val = pfcp->FOLR;
    reg_val |= (PFCP_FOLR_FOLKEY << PFCP_FOLR_FOLKEY_Pos);
    reg_val &= ~(0x01UL << PFCP_FOLR_SWFO_Pos);
    reg_val &= ~(0x01UL << PFCP_FOLR_PRT0FO_Pos);
    reg_val &= ~(0x01UL << PFCP_FOLR_PRT1FO_Pos);
    pfcp->FOLR = reg_val;

    return DRIVER_ERROR_OK;
}

int32_t HAL_PFCP_DisablePWMOutput(enum pfcp_id id)
{
    uint32_t reg_val;
    PFCP_Type *pfcp;

    DRIVER_ASSERT(id < PFCP_ID_MAX);

    pfcp = PFCP_REG(id);

    reg_val = pfcp->FOLR;
    reg_val |= (PFCP_FOLR_FOLKEY << PFCP_FOLR_FOLKEY_Pos);
    reg_val |= (0x01UL << PFCP_FOLR_SWFO_Pos);
    reg_val |= (0x01UL << PFCP_FOLR_PRT0FO_Pos);
    reg_val |= (0x01UL << PFCP_FOLR_PRT1FO_Pos);
    pfcp->FOLR = reg_val;

    return DRIVER_ERROR_OK;
}

void PFCPPROT0_IRQHandler(void)
{
    struct pfcp_ctrl_block *mcb;

    mcb = &g_mcb[PFCP_ID_0];

    if (mcb->handler)
    {
        mcb->handler(PFCP_IRQ_EVENT_TYPE_PRT0, mcb->context);
    }
}

void PFCPPROT1_IRQHandler(void)
{
    struct pfcp_ctrl_block *mcb;

    mcb = &g_mcb[PFCP_ID_0];

    if (mcb->handler)
    {
        mcb->handler(PFCP_IRQ_EVENT_TYPE_PRT1, mcb->context);
    }
}

void PFCP_IRQHandler(void)
{
    struct pfcp_ctrl_block *mcb;

    mcb = &g_mcb[PFCP_ID_0];

    if (mcb->handler)
    {
        mcb->handler(PFCP_IRQ_EVENT_TYPE_PFCP, mcb->context);
    }
}



